/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var assert = require('assert');
var fs = require('fs');
var path = require('path');
var adapterCreate = require('../lib/create.js');
var adapterBuild = require('../lib/build.js');
var adapterDeploy = require('../lib/deploy.js');
var adapterPull = require('../lib/pull.js');
var adapterPush = require('../lib/push.js');
var validateAdapterName = adapterCreate.validateAdapterName;
var validateAdapterType = adapterCreate.validateAdapterType;
var validatePackageName = adapterCreate.validatePackageName;
var validateGroupID = adapterCreate.validateGroupID;
var errors = require('mdo-errors');
var originalCWD = process.cwd();

describe('mdo-adapter-actions: create', function () {
  before(function(done) {
    fs.mkdirSync('existingAdapter');
    done();
  });

  after(function(done) {
    var stat = fs.statSync('existingAdapter');
    if (stat.isDirectory()) fs.rmdirSync('existingAdapter');
    done();
  });

  it('Missing adapter name', function () {
    var expectedError = errors.getError("MISSINGNAME");
    var actualError = validateAdapterName(null, true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Invalid adapter name', function () {
    var expectedError = errors.getError("INVALIDNAME");
    var actualError = validateAdapterName('123abc', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Existing adapter name', function () {
    var expectedError = errors.getError("FILEEXISTS");
    var actualError = validateAdapterName('existingAdapter', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Missing adapter type', function () {
    var expectedError = errors.getError("MISSINGADAPTERTYPE");
    var actualError = validateAdapterType(null, true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Invalid adapter type', function () {
    var expectedError = errors.getError("INVALIDADAPTERTYPE");
    var actualError = validateAdapterType('invalidType', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Missing package name', function () {
    var expectedError = errors.getError("MISSINGPACKAGENAME");
    var actualError = validatePackageName(null, 'java', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Invalid package period', function () {
    var expectedError = errors.getError("INVALIDPACKAGEPERIOD");
    var actualError = validatePackageName('.testPackage', 'java', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Package only valid for Java adapters', function () {
    var expectedError = errors.getError("CANNOTADDPACKAGENAME");
    var actualError = validatePackageName('testPackage', 'jms', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Invalid package name', function () {
    var expectedError = errors.getError("INVALIDNAME");
    var actualError = validatePackageName('123abc', 'java', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Missing group ID', function () {
    var expectedError = errors.getError("MISSINGGROUPID");
    var actualError = validateGroupID(null, true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Invalid group ID period', function () {
    var expectedError = errors.getError("INVALIDGROUPPERIOD");
    var actualError = validateGroupID('.testGroup', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });

  it('Invalid group ID', function () {
    var expectedError = errors.getError("INVALIDID");
    var actualError = validateGroupID('123abc', true);
    assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
  });
});

describe('mdo-adapter-actions: build', function() {
  it('Invalid adapter project', function(done) {
    var expectedError = errors.getError("NONADAPTERFOLDER");
    adapterBuild.run().then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });
});

describe('mdo-adapter-actions: deploy', function() {
  after(function(done) {
    process.chdir(originalCWD);
    done();
  });

  it('Invalid deploy params', function(done) {
    var expectedError = errors.getError("INVALIDADAPTERPARAMS");
    var argv = ["localhost", "arg2", "arg3"];

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterDeploy.run(argv).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });
});

describe('mdo-adapter-actions: deploy all', function() {
  after(function(done) {
    process.chdir(originalCWD);
    done();
  });

  it('Invalid deploy params', function(done) {
    var expectedError = errors.getError("INVALIDADAPTERPARAMS");
    var argv = ["all", "arg1", "arg2", "arg3"];

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterDeploy.run(argv).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });
});

describe('mdo-adapter-actions: pull', function() {
  after(function(done) {
    process.chdir(originalCWD);
    done();
  });

  it('Invalid pull params', function(done) {
    var expectedError = errors.getError("INVALIDADAPTERPARAMS");
    var argv = ["localhost", "arg2", "arg3"];

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterPull.run(argv).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });

  it('Fail to pull', function(done) {
    var expectedError = errors.getError("MISSINGSERVERDEFINITION");

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterPull.run(['']).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });

  it('Verify configFile flag is accepted - pull will still fail', function(done) {
    var expectedError = errors.getError("MISSINGSERVERDEFINITION");

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterPull.run(["--configFile", "config.json"]).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });
});

describe('mdo-adapter-actions: push', function() {
  after(function(done) {
    process.chdir(originalCWD);
    done();
  });

  it('Invalid push params', function(done) {
    var expectedError = errors.getError("INVALIDADAPTERPARAMS");
    var argv = ["localhost", "arg2", "arg3"];

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterPush.run(argv).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });

  it('Fail to push', function(done) {
    var expectedError = errors.getError("MISSINGSERVERDEFINITION");

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterPush.run(['']).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });

  it('Verify configFile flag is accepted - push will still fail', function(done) {
    var expectedError = errors.getError("MISSINGSERVERDEFINITION");

    process.chdir(path.resolve(__dirname, 'resources', 'dummyAdapter'));

    adapterPush.run(["--configFile", "config.json"]).then(function() {
      assert(false, 'Promise should be rejected');
    }).fail(function(actualError) {
      assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
    }).done(function() {
      done();
    });
  });
});

function checkError(expectedError, actualError) {
  return expectedError && actualError ? expectedError.errno === actualError.errno : false;
}
