/*
	Licensed Material - Property of IBM

	(C) Copyright 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach, afterEach */

var assert = require('assert');
var fs = require('fs');
var path = require('path');
var http = require('http');
var httpserver = null;
var adapterCall = require('../lib/call.js');
var errors = require('mdo-errors');

var server = require('../../mdo-cmd-server/lib/index.js');
global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '../../mfpdev-cli/node_modules');
global.MDO_CLI_FULL_VERSION = require('../../mfpdev-cli/package.json').version + '_mdo-adapter-actions';

var HOME = process.env[(process.platform === 'win32') ? 'USERPROFILE' : 'HOME'];
var IBM_HOME = path.join(HOME, '.ibm');
var MFP_HOME = path.join(IBM_HOME, 'mobilefirst');
var versionPath = path.join(MFP_HOME, global.MDO_CLI_FULL_VERSION);
var profilePath = path.join(versionPath, 'server_profiles.json');

describe('mdo-adapter-actions: call', function() {
  beforeEach(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }
    stopServer();
    done();
  });

  afterEach(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }
    stopServer();
    done();
  });

  after(function(done) {
    if (fs.existsSync(versionPath)) {
      try {
        fs.rmdirSync(versionPath);
      }
      catch (err) {
        assert(false, 'cannot delete directory: ' + versionPath);
      }
    }
    done();
  });

  it('Profile not found', function(done) {
    this.timeout(10000);
    var expectedError = errors.getError("PROFILENOTFOUND");
    server.run(['add', 'remoteserver', '-u', 'http://remoteserver.ibm.com:9080', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['adapter/procedure', '--server', 'local'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function(){
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('Entered runtime without server', function(done) {
    var expectedError = errors.getError("RUNTIMEWITHOUTSERVER");
    server.run(['add', 'remoteserver', '-u', 'http://remoteserver.ibm.com:9080', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['adapter/procedure', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function(){
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('No runtimes on server', function(done) {
    var expectedError = errors.getError("NORUNTIMESEXIST");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify( {
             "projects" : {}
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['adapter/procedure', '--server', 'remoteserver'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('Adapter not found', function(done) {
    var expectedError = errors.getError("ADAPTERNOTFOUND");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify( {
             "projects" : [{"name":"mfp"}]
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['adapter/procedure', '--server', 'remoteserver', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('Adapter not found on server', function(done) {
    var expectedError = errors.getError("ADAPTERNOTFOUND");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
             "projects" : [{"name":"mfp"}],
             "items":[{ "resourceType": 'ADAPTER_CONTENT',
                        "name": 'test_adapter' }]
          })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['adapter/procedure', '--server', 'remoteserver', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('JSON file does not exist', function(done) {
    var expectedError = errors.getError("FILENOEXISTS");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
             "projects" : [{"name":"mfp"}],
             "items":[{ "resourceType": 'ADAPTER_CONTENT',
                        "name": 'test_adapter' }],
             "runtimeInfo": {
               "resources": {"paths": {}},
               "descriptor": {"procedures": {}}
             }
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['test_adapter/procedure', '--server', 'remoteserver', '--runtime', 'mfp', '--file', 'noexist.json'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('Invalid JSON parameters', function(done) {
    var expectedError = errors.getError("INVALIDJSON");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
             "projects" : [{"name":"mfp"}],
             "items": [{ "resourceType": 'ADAPTER_CONTENT',
                         "name": 'test_adapter' }],
             "runtimeInfo": {
               "resources": {"paths": {}},
               "descriptor": {"procedures": {"procedure":{"scope":'',"secured":"true"}}}
             }
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['test_adapter/procedure', 'string', '--server', 'remoteserver', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('Bad arguments', function(done) {
    var expectedError = errors.getError("BADARGUMENTS");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
             "projects" : [{"name":"mfp"}],
             "items": [{ "resourceType": 'ADAPTER_CONTENT',
                         "name": 'test_adapter' }],
             "runtimeInfo": {
               "resources": {"paths": {}},
               "descriptor": {"procedures": {}}
             }
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['test_adapter/procedure', 'string', '--server', 'remoteserver', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      done();
    });
  });

  it('Invalid Call Format', function(done) {
    var expectedError = errors.getError("CALLFORMAT");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
             "projects" : [{"name":"mfp"}],
             "items": [{ "resourceType": 'ADAPTER_CONTENT',
                         "name": 'test_adapter' }],
             "runtimeInfo": {
               "resources": {"paths": {}},
               "descriptor": {"procedures": {}}
             }
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['test_adapter/', '--server', 'remoteserver', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });

  it('Invalid Java adapter parameters', function(done) {
    var expectedError = errors.getError("INVALIDPARAM");
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
             "projects" : [{"name":"mfp"}],
             "items": [{ "resourceType": 'ADAPTER_CONTENT',
                         "name": 'test_adapter' }],
             "runtimeInfo": {
               "resources": {"paths": {}},
               "descriptor": {"procedures": {}}
             }
           })
       });
    server.run(['add', 'remoteserver', '-u', 'http://localhost:10088', '-l', 'demo', '-p', 'demo', '-t', '0'])
    .then(function() {
      var argv = ['test_adapter/procedure', '--query', 'invalid', '--form', 'invalid', '--server', 'remoteserver', '--runtime', 'mfp'];

      adapterCall.run(argv).then(function() {
        assert(false, 'Promise should be rejected');
      }).fail(function(actualError) {
        assert(checkError(expectedError, actualError), 'Expected ' + expectedError + ', Actual ' + actualError);
      }).done(function() {
        done();
      });
    }).catch(function() {
      assert(false, 'Failed to add remoteserver.');
      done();
    });
  });
  
  // Starts a HTTP server used to simulate a specific HTTP response for all requests
  //
  // Parameter - single JSON object in format:
  // {
  //   port: <listen port>
  //   respHttpCode: <response HTTP status code>
  //   respHttpHeader: <JSON object.  Each attribute/value is a response header/value pair>
  //   respContent: <string or buffer containing response body>
  // }
  function startServer(options) {
      httpserver = http.createServer( function(req, res) {
          res.writeHead(options.respHttpCode, options.respHttpHeader);
          res.end(options.respContent);
      }).listen(options.port);
  }

  function stopServer() {
      if (httpserver) {
          httpserver.close();
          httpserver = null;
      }
  }
});

function checkError(expectedError, actualError) {
  return expectedError && actualError ? expectedError.errno === actualError.errno : false;
}
