/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var fs = require('fs');
var path = require('path');
var propertiesParser = require('properties-parser');
//private packages
var androidPlatform = require('../../lib/nativeAndroid');
var getTestFixture = require('mdo-test-resources');

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

var fixturePath,
  cordovaProject,
  androidProjPath,
  propertyFilePath;

describe('platform-native-android process properties files', function () {
  before(function() {
    fixturePath = path.resolve(__dirname, '..', 'fixtures', 'processPropertyFile');
    cordovaProject = getTestFixture('AndroidNativeProject.zip', fixturePath);

    cordovaProject.clean();
    androidProjPath = path.resolve(fixturePath, 'AndroidNativeProject');

    propertyFilePath = path.join(androidProjPath, 'app', 'src', 'main', 'assets', 'mfpclient.properties');
  });

  after(function() {
    cordovaProject.delete();
  });

  it('should build mfpclient.properties when not present', function (done) {
      assert.deepEqual(validateFileOrFolderExists(androidProjPath), true, 'AndroidNativeProject should exist');
      assert.deepEqual(validateFileOrFolderExists(propertyFilePath), false, 'mfpclient.properties should not exist yet');

      var platformObj = {platformPath: androidProjPath};
      var serverObj = {
          protocol: 'http',
          host: '9.12.34.56',
          port: '5050',
      };
      var runtime = 'myRuntime';

      androidPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(validateFileOrFolderExists(propertyFilePath), true, 'mfpclient.properties should exist now');
        var mfpClientFile = fs.readFileSync(propertyFilePath);
        var mfpClientData = propertiesParser.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['wlServerProtocol'], 'http');
        assert.deepEqual(mfpClientData['wlServerHost'], '9.12.34.56');
        assert.deepEqual(mfpClientData['wlServerPort'], '5050');
        assert.deepEqual(mfpClientData['wlServerContext'], '/myRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update mfpclient.properties when file exists and server info is different', function (done) {
      assert.deepEqual(validateFileOrFolderExists(androidProjPath), true, 'AndroidNativeProject should exist');
      assert.deepEqual(validateFileOrFolderExists(propertyFilePath), true, 'mfpclient.properties should exist');

      var platformObj = {platformPath: androidProjPath};
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: '9090',
      };
      var runtime = 'altRuntime';

      androidPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(validateFileOrFolderExists(propertyFilePath), true, 'mfpclient.properties should exist now');
        var mfpClientFile = fs.readFileSync(propertyFilePath);
        var mfpClientData = propertiesParser.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['wlServerProtocol'], 'https');
        assert.deepEqual(mfpClientData['wlServerHost'], '1.2.3.4.5');
        assert.deepEqual(mfpClientData['wlServerPort'], '9090');
        assert.deepEqual(mfpClientData['wlServerContext'], '/altRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update port when it is a Number', function (done) {
      assert.deepEqual(validateFileOrFolderExists(androidProjPath), true, 'AndroidNativeProject should exist');
      assert.deepEqual(validateFileOrFolderExists(propertyFilePath), true, 'mfpclient.properties should exist');

      var platformObj = {platformPath: androidProjPath};
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: 9010,
      };
      var runtime = 'altRuntime';

      androidPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(validateFileOrFolderExists(propertyFilePath), true, 'mfpclient.properties should exist now');
        var mfpClientFile = fs.readFileSync(propertyFilePath);
        var mfpClientData = propertiesParser.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['wlServerProtocol'], 'https');
        assert.deepEqual(mfpClientData['wlServerHost'], '1.2.3.4.5');
        assert.deepEqual(mfpClientData['wlServerPort'], '9010');
        assert.deepEqual(mfpClientData['wlServerContext'], '/altRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        done();
      }).catch(function (err) {
        done(err);
      });
  });
});
