/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var assert = require('assert');
var appConfig = require('../lib/index');
var Q = require('q');
var errors = require('mdo-errors');
var path = require('path');
var testData = require('./test-data');
var testResources = require('mdo-test-resources');
var fs = require('fs');

// Set global variables so that mdo extensions, server stuff works
global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname,
    '../../mfpdev-cli/node_modules');
global.MDO_CLI_FULL_VERSION = require('../../mfpdev-cli/package.json').version +
                         '_mdo-app-config';
var HOME = process.env[(process.platform === 'win32') ? 'USERPROFILE' : 'HOME'];
var IBM_HOME = path.join(HOME, '.ibm');
var MFP_HOME = path.join(IBM_HOME, 'mobilefirst');
var versionPath = path.join(MFP_HOME, global.MDO_CLI_FULL_VERSION);
var profilePath = path.join(versionPath, 'server_profiles.json');

var FIXTURES_PATH = path.resolve(__dirname, 'fixtures');

function errorsEqual(err1, err2) {
  return err1 && err2 ? err1.errno === err2.errno : false;
}

function runProjectTest(projectZipName, projectRelativePath, testData,
                        propertiesRelativePath) {
  var fixture;
  var projectPath;
  var propertiesPath;
  var origCwd;

  function unzipProject() {
    projectPath = path.resolve(FIXTURES_PATH, projectRelativePath);
    if (propertiesRelativePath)
      propertiesPath = path.resolve(projectPath, propertiesRelativePath);
    fixture = testResources(projectZipName, FIXTURES_PATH);
    fixture.clean();
  }

  describe('mdo-app-config ' + projectRelativePath, function() {
    before(function(done) {
      unzipProject();
      origCwd = process.cwd();
      done();
    });

    after(function(done) {
      fixture.delete();
      process.chdir(origCwd);
      done();
    });

    it('\'app config --list\' returns initial values', function(done) {
      process.chdir(projectPath);
      appConfig.run(['--list']).then(function(settings) {
        var expectedNumKeys = Object.keys(testData.keys).length;
        // Assert all keys in testData are returned, and that their values are
        // equal to initialValue
        for (var key in testData.keys) {
          var actualValue = settings[key];
          var expectedValue = testData.keys[key].initialValue;
          if (!(key in testData.keys)) {
            assert(false, 'configSettings contains ' + key);
          } else {
            assert(actualValue === expectedValue,
                   'configSettings contains ' + expectedValue + ' at ' + key);
          }
        }
        // Assert no extra keys were returned
        assert(Object.keys(settings).length === expectedNumKeys,
               'Number of app config settings is ' + expectedNumKeys);
      }, function(err) {
        assert(false, 'Got error :' + JSON.stringify(err));
      }).done(function() {
        done();
      });
    });

    it('\'app config <key>\' returns initial value', function(done) {
      process.chdir(projectPath);
      var promises = Object.keys(testData.keys).map(function(key) {
        return function() {
          var d = Q.defer();
          appConfig.run([key]).then(function(resp) {
            assert(resp === testData.keys[key].initialValue,
                   'Value returned is ' + testData.keys[key].initialValue);
          }, function() {
            assert(false, 'Promise should not be rejected');
          }).done(d.resolve);
          return d.promise;
        };
      });

      // Run commands sequentially
      promises.reduce(Q.when, new Q()).done(function() {
        done();
      });
    });

    it('\'app config <invalid key>\' fails', function(done) {
      process.chdir(projectPath);
      var invalidKeyError = errors.getError('INVALID_APP_CONFIG_KEY');
      var promises = testData.invalidKeys.map(function(invalidKey) {
        // Return a function that tries setting an invalid key
        return function() {
          var d = Q.defer();
          appConfig.run([invalidKey]).then(function() {
            assert(false, 'Promise should be rejected');
          }, function(error) {
            assert(errorsEqual(error, invalidKeyError),
                   'Error is ' + JSON.stringify(invalidKeyError));
          }).done(d.resolve);
          return d.promise;
        };
      });

      // Run commands sequentially
      promises.reduce(Q.when, new Q()).done(function() {
        done();
      });
    });

    it('\'app config <key> <invalid value>\' fails', function(done) {
      process.chdir(projectPath);
      var invalidValueError = errors.getError('INVALID_APP_CONFIG_VALUE');
      var promises = Object.keys(testData.keys).reduce(function(promises, key) {
        var invalidValues = testData.keys[key].invalidValues;

        return promises.concat(invalidValues.map(function(invalidValue) {
          // Return a function that tries setting a key to an invalid value
          return function() {
            var d = Q.defer();
            appConfig.run([key, invalidValue]).then(function() {
              assert(false, 'Promise should be rejected');
            }, function(error) {
              assert(errorsEqual(error, invalidValueError),
                     'Error is ' + JSON.stringify(invalidValueError));
            }).done(d.resolve);
            return d.promise;
          };
        }));
      }, []);

      // Run commands sequentially
      promises.reduce(Q.when, new Q()).done(function() {
        done();
      });
    });

    it('\'app config <key>\' confirms value did not change', function(done) {
      process.chdir(projectPath);
      var promises = Object.keys(testData.keys).map(function(key) {
        return function() {
          var d = Q.defer();
          appConfig.run([key]).then(function(resp) {
            assert(resp === testData.keys[key].initialValue,
                   'Value returned is ' + testData.keys[key].initialValue);
          }, function() {
            assert(false, 'Promise should not be rejected');
          }).done(d.resolve);
          return d.promise;
        };
      });

      // Run commands sequentially
      promises.reduce(Q.when, new Q()).done(function() {
        done();
      });
    });

    it('\'app config <key> <value>\' resolves, then ' +
       '\'app config <key>\' confirms the value changed', function(done) {
      process.chdir(projectPath);
      var promises = Object.keys(testData.keys).reduce(function(promises, key) {
        var validValues = testData.keys[key].validValues;

        return promises.concat(validValues.reduce(function(promises, value) {
          if (!value) {
            return promises;
          } else {
            var setValue = value.set;
            var getValue = value.get;

            // Concatenate to promises a function setting the key and a function
            // getting the key
            return promises.concat([
              function() {
                // app config <key> <setValue>, confirm promise was resolved
                var d = Q.defer();
                appConfig.run([key, setValue]).then(function() {
                  assert(true, 'Promise was resolved');
                }, function() {
                  assert(false, 'Promise was resolved');
                }).done(d.resolve);
                return d.promise;
              }, function() {
                // app config <key>, confirm value is now getValue
                var d = Q.defer();
                appConfig.run([key]).then(function(resp) {
                  if (typeof(getValue) === 'function') {
                    // getValue is a function that validates resp
                    assert(getValue(resp), 'Value returned is valid');
                  } else {
                    assert(resp === getValue,
                        'Value returned is ' + getValue);
                  }
                }, function() {
                  assert(false, 'Promise should not be rejected');
                }).done(d.resolve);
                return d.promise;
              }
            ]);
          }
        }, []));
      }, []);

      // Run commands sequentially
      promises.reduce(Q.when, new Q()).done(function() {
        done();
      });
    });

    // Running the command with three arguments fails
    it('\'app config arg1 arg2 arg3\' fails', function(done) {
      process.chdir(projectPath);
      var expectedError = errors.getError('BADARGUMENTS');
      appConfig.run(['arg', 'arg', 'arg']).then(function() {
        assert(false, 'Promise should be rejected');
      }, function(error) {
        assert(errorsEqual(error, expectedError), 'Error is ' + error);
      }).done(function() {
        done();
      });
    });

    // Delete the properties file, confirm we get error if no config file
    // is present
    it('\'app config\' fails when no properties file', function(done) {
      process.chdir(projectPath);
      var expectedError = errors.getError(testData.errorWhenNoPropertiesFile);
      if (propertiesPath) {
        fs.unlink(propertiesPath, function(err) {
          if (err) {
            assert(false, 'Unable to delete properties file');
            done();
          } else {
            // When the config file goes missing, the error differs depending on
            // the platform, so we have errorWhenNoPropertiesFile in testData
            appConfig.run([]).then(function() {
              assert(false, 'Promise should be rejected');
            }, function(error) {
              assert(errorsEqual(error, expectedError), 'Error is ' + error);
            }).done(function() {
              done();
            });
          }
        });
      } else {
        done();
      }
    });
  });
}

describe('mdo-app-config', function() {
  var profileFixture;
  var origCwd;

  before(function(done) {
    // Save original cwd
    origCwd = process.cwd();

    // Create file for server profile
    // Create directories up to profilePath
    if (!fs.existsSync(IBM_HOME)) fs.mkdirSync(IBM_HOME);
    if (!fs.existsSync(MFP_HOME)) fs.mkdirSync(MFP_HOME);
    if (!fs.existsSync(versionPath)) fs.mkdirSync(versionPath);

    // Unzip our server_profiles
    profileFixture = testResources('server_profiles.json.zip', FIXTURES_PATH);
    profileFixture.clean();

    // Read our example server_profiles.json from mdo-test-resources
    var serverProfilesPath = path.join(FIXTURES_PATH, 'server_profiles.json');
    var serverJSON = fs.readFileSync(serverProfilesPath).toString();

    // Write our example server_profiles.json to profilePath
    fs.writeFileSync(profilePath, serverJSON);
    done();
  });

  after(function(done) {
    // Delete profilePath, profileFixture after we're done
    fs.unlinkSync(profilePath);
    profileFixture.delete();
    // cd back to original cwd
    process.chdir(origCwd);
    done();
  });

  runProjectTest('AndroidNativeProjectAppConfig.zip',
                 'AndroidNativeProjectAppConfig',
                 testData.androidNativeTestData,
                 path.join('app', 'src', 'main', 'assets',
                           'mfpclient.properties'));

  runProjectTest('IOSNativeProjectAppConfig.zip',
                 'IOSNativeProjectAppConfig',
                 testData.iOSNativeTestData,
                 path.join('mfpclient.plist'));

  runProjectTest('IOSNativeProjectNonDefaultPlistLocationAppConfig.zip',
                 'IOSNativeProjectNonDefaultPlistLocationAppConfig',
                 testData.iOSNativeTestData,
                 path.join('TestNativeProject', 'folderContainingPlist',
                           'mfpclient.plist'));

  runProjectTest('Windows10NativeProjectAppConfig.zip',
                 path.join('Windows10NativeProjectAppConfig', 'App2'),
                 testData.windows10NativeTestData,
                 path.join('strings', 'mfpclient.resw'));

  runProjectTest('Windows8NativeProjectAppConfig.zip',
                 path.join('Windows8NativeProjectAppConfig', 'BlankSharedApp1',
                           'BlankSharedApp1', 'BlankSharedApp1.Windows'),
                 testData.windows8NativeTestData,
                 path.join('strings', 'mfpclient.resw'));

  runProjectTest('Windows8NativeProjectAppConfig.zip',
                 path.join('Windows8NativeProjectAppConfig', 'BlankSharedApp1',
                           'BlankSharedApp1', 'BlankSharedApp1.WindowsPhone'),
                 testData.windowsPhone8NativeTestData,
                 path.join('strings', 'mfpclient.resw'));

  runProjectTest('CordovaProject.zip',
                 'CordovaProject',
                 testData.cordovaAllPlatformsData,
                 'config.xml');

  runProjectTest('CordovaOnlyIOSProject.zip',
                 'CordovaOnlyIOSProject',
                 testData.cordovaOnlyIOSData,
                 'config.xml');

  runProjectTest('CordovaProjectNoPlatforms.zip',
                 'CordovaProjectNoPlatforms',
                 testData.cordovaNoPlatformsData,
                 'config.xml');

  describe('mdo-app-config miscellaneous testing', function() {
    it('\'app config\' outside of a project fails', function(done) {
      process.chdir(FIXTURES_PATH); // not a project
      var expectedError = errors.getError('NOPLATFORMSDETECTED');
      appConfig.run([]).then(function() {
        assert(false, 'Promise should be rejected');
      }, function(error) {
        assert(errorsEqual(error, expectedError), 'Error is ' + error);
      }).done(function() {
        done();
      });
    });
  });
});
