'use strict';

var assert = require('assert');
var path = require('path');
var findExtensions = require('pluggington');
var mdoApp = require('../lib/index.js');
var rimraf = require('rimraf');
var fs = require('fs');

describe('mdo-app-create', function() {
  this.timeout(50000);

  before(function() {
    global.MDO_MAIN_EXTENSIONS_PATH =
      path.join(__dirname, '../../mfpdev-cli/node_modules');
  });

  it('should contribute a mdo.cli.app.actions extension', function() {
    var extensions = findExtensions(path.resolve(__dirname, '..', '..'), 'mdo.cli.app.actions');

    assert.notEqual(extensions, undefined, 'There should be a mdo.cli.app.actions extension here');

    var foundCommand = false;
    extensions.forEach(function(commandExtension) {
      if (commandExtension.name === "create") {
        foundCommand = true;
      }
    });

    assert.equal(foundCommand, true, "I didn't find the app command extension!");
  });

  it('should find a valid mdo.cli.app.create extension', function() {
    var extensions = findExtensions(__dirname, 'mdo.cli.app.create');

    assert.notEqual(extensions, undefined, 'There should be a mdo.cli.app.create extension here');

    var foundExtension = false;
    extensions.forEach(function(extension) {
      if (extension.name === "GoodExtension") {
        foundExtension = true;

        assert.notEqual(extension.export.run, undefined, "Did not find a run function in this extension");
      }
    });

    assert.equal(foundExtension, true, "I didn't find the app action extension!");
  });

  it('should throw error as apptype is missing', function(done) {
    mdoApp.run(['TestApp']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 205, 'Expected return code 205');
      done();
    }).catch(function (err) {
      done(err);
    });
  });

  it('should throw error as apptype is invalid', function(done) {
    mdoApp.run(['TestApp','-t', 'kk']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 205, 'Expected return code 205');
      done();
    }).catch(function (err) {
      done(err);
    });
  }); 
  
  it('should throw error as app name is invalid', function(done) {
    mdoApp.run(['TestApp%.','-t', 'ionic']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 206, 'Expected return code 206');
      done();
    }).catch(function (err) {
      done(err);
    });
  });    

  it('should throw error as bundleId is missing', function(done) {
    mdoApp.run(['TestApp','-t', 'ionic']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 207, 'Expected return code 207');
      done();
    }).catch(function (err) {
      done(err);
    });
  });  

  it('should throw error as bundleId is invalid', function(done) {
    mdoApp.run(['TestApp','-t', 'ionic', '-i', 'com.aa.']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 208, 'Expected return code 208');
      done();
    }).catch(function (err) {
      done(err);
    });
  });

  it('should throw error as platform is invalid', function(done) {
    mdoApp.run(['TestApp','-t', 'ionic', '-i', 'com.aa', '-p', 'xyz']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 210, 'Expected return code 210');
      done();
    }).catch(function (err) {
      done(err);
    });
  }); 
  
  it('should throw error as version is missing', function(done) {
    mdoApp.run(['TestApp','-t', 'ionic', '-i', 'com.aa', '-p', 'android']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 211, 'Expected return code 211');
      done();
    }).catch(function (err) {
      done(err);
    });
  }); 

  it('should throw error as version is invalid', function(done) {
    mdoApp.run(['TestApp','-t', 'ionic', '-i', 'com.aa', '-p', 'android', '-av', 'aa']).then(function () {
      assert(false, 'error should have been thrown');
    }).fail(function (err) {
      assert.strictEqual(err.errno, 212, 'Expected return code 212');
      done();
    }).catch(function (err) {
      done(err);
    });
  });  

  it('create ionic app',  function(done) {
    mdoApp.run(['TestApp','-t', 'ionic', '-i', 'com.aa', '-p', 'android', '-av', '1.0.0']).then(function () {
      assert(fs.existsSync(path.join('.', 'TestApp')), "Ionic app creation failed");
      assert(fs.existsSync(path.join('.', 'TestApp', 'ionic', 'config.xml')), "Not an ionic app as config.xml is missing");
      assert(fs.existsSync(path.join('.', 'TestApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");      
      assert(!fs.existsSync(path.join('.', 'TestApp', 'mfpconfig', 'server_config.json')), "server_config.json should not be created");
      cleanupFile(path.join('.', 'TestApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'TestApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'TestApp'));
      done(err);
    });
  });  

  it('create native android app',  function(done) {
    mdoApp.run(['TestApp','-t', 'native', '-i', 'com.aa', '-p', 'android', '-av', '1.0.0']).then(function () {
      assert(fs.existsSync(path.join('.', 'TestApp')), "Native android app creation failed");
      assert(fs.existsSync(path.join('.', 'TestApp', 'app', 'src', 'main', 'AndroidManifest.xml')), "Not an android app as manifest is missing");
      assert(fs.existsSync(path.join('.', 'TestApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");      
      assert(!fs.existsSync(path.join('.', 'TestApp', 'mfpconfig', 'server_config.json')), "server_config.json should not be created");
      cleanupFile(path.join('.', 'TestApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'TestApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'TestApp'));
      done(err);
    });
  }); 

  it('create native iOS app',  function(done) {
    mdoApp.run(['TestApp','-t', 'native', '-i', 'com.aa', '-p', 'ios', '-av', '1.0.0']).then(function () {
      assert(fs.existsSync(path.join('.', 'TestApp')), "Native iOS app creation failed");
      assert(fs.existsSync(path.join('.', 'TestApp', 'TestApp.xcodeproj', 'project.pbxproj')), "Not an ios app as pbxproj is missing");
      assert(fs.existsSync(path.join('.', 'TestApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");
      assert(!fs.existsSync(path.join('.', 'TestApp', 'mfpconfig', 'server_config.json')), "server_config.json should not be created");
      cleanupFile(path.join('.', 'TestApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'TestApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'TestApp'));
      done(err);
    });
  }); 

  it('create ionic app using app config',  function(done) {
    mdoApp.run(['-ac', './test/artifacts/app_config_ionic.json']).then(function () {
      assert(fs.existsSync(path.join('.', 'StarterIonicApp')), "Ionic app creation failed");
      assert(fs.existsSync(path.join('.', 'StarterIonicApp', 'ionic', 'config.xml')), "Not an ionic app as config.xml is missing");
      assert(fs.existsSync(path.join('.', 'StarterIonicApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");   
      assert(!fs.existsSync(path.join('.', 'StarterIonicApp', 'mfpconfig', 'server_config.json')), "server_config.json should not be created");   
      cleanupFile(path.join('.', 'StarterIonicApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'StarterIonicApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'StarterIonicApp'));
      done(err);
    });
  });   
  
  it('create native android app using app config',  function(done) {
    mdoApp.run(['-ac', './test/artifacts/app_config_android.json']).then(function () {
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp')), "Native android app creation failed");
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp', 'app', 'src', 'main', 'AndroidManifest.xml')), "Not an android app as manifest is missing");
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");    
      assert(!fs.existsSync(path.join('.', 'StarterAndroidApp', 'mfpconfig', 'server_config.json')), "server_config.json should not be created");  
      cleanupFile(path.join('.', 'StarterAndroidApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'StarterAndroidApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'StarterAndroidApp'));
      done(err);
    });
  });   
  
  it('create native iOS app using app config',  function(done) {
    mdoApp.run(['-ac', './test/artifacts/app_config_ios.json']).then(function () {
      assert(fs.existsSync(path.join('.', 'StarterIosApp')), "Native iOS app creation failed");
      assert(fs.existsSync(path.join('.', 'StarterIosApp', 'StarterIosApp.xcodeproj', 'project.pbxproj')), "Not an ios app as pbxproj is missing");
      assert(fs.existsSync(path.join('.', 'StarterIosApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");
      assert(!fs.existsSync(path.join('.', 'StarterIosApp', 'mfpconfig', 'server_config.json')), "server_config.json should not be created");
      cleanupFile(path.join('.', 'StarterIosApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'StarterIosApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'StarterIosApp'));
      done(err);
    });
  });   

  it('create ionic app using app and server config',  function(done) {
    mdoApp.run(['-ac', './test/artifacts/app_config_ionic.json', '-sc', './test/artifacts/server_config.json']).then(function () {
      assert(fs.existsSync(path.join('.', 'StarterIonicApp')), "Ionic app creation failed");
      assert(fs.existsSync(path.join('.', 'StarterIonicApp', 'ionic', 'config.xml')), "Not an ionic app as config.xml is missing");
      assert(fs.existsSync(path.join('.', 'StarterIonicApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");   
      assert(fs.existsSync(path.join('.', 'StarterIonicApp', 'mfpconfig', 'server_config.json')), "server_config.json is not created");   
      cleanupFile(path.join('.', 'StarterIonicApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'StarterIonicApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'StarterIonicApp'));
      done(err);
    });
  });   
  
  it('create native android app using app and server config',  function(done) {
    mdoApp.run(['-ac', './test/artifacts/app_config_android.json', '-sc', './test/artifacts/server_config.json']).then(function () {
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp')), "Native android app creation failed");
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp', 'app', 'src', 'main', 'AndroidManifest.xml')), "Not an android app as manifest is missing");
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");    
      assert(fs.existsSync(path.join('.', 'StarterAndroidApp', 'mfpconfig', 'server_config.json')), "server_config.json is not created");  
      cleanupFile(path.join('.', 'StarterAndroidApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'StarterAndroidApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'StarterAndroidApp'));
      done(err);
    });
  });   
  
  it('create native iOS app using app and server config',  function(done) {
    mdoApp.run(['-ac', './test/artifacts/app_config_ios.json', '-sc', './test/artifacts/server_config.json']).then(function () {
      assert(fs.existsSync(path.join('.', 'StarterIosApp')), "Native iOS app creation failed");
      assert(fs.existsSync(path.join('.', 'StarterIosApp', 'StarterIosApp.xcodeproj', 'project.pbxproj')), "Not an ios app as pbxproj is missing");
      assert(fs.existsSync(path.join('.', 'StarterIosApp', 'mfpconfig', 'app_config.json')), "app_config.json is not created");
      assert(fs.existsSync(path.join('.', 'StarterIosApp', 'mfpconfig', 'server_config.json')), "server_config.json is not created");
      cleanupFile(path.join('.', 'StarterIosApp'));
      done();
    }).fail(function (err) {
      cleanupFile(path.join('.', 'StarterIosApp'));
      done(err);
    }).catch(function (err) {
      cleanupFile(path.join('.', 'StarterIosApp'));
      done(err);
    });
  });
});

function cleanupFile(file) {
  rimraf.sync(file);
}
