/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var assert = require('assert');
var path = require('path');
var fs = require('fs');

var consts = require('../lib/preview-consts');
var PreviewWatch = require('../lib/preview-watch');
var testFixtures = require('mdo-test-resources');
var cordovaSupport = require('mdo-cordova-support');
var exec = require('mdo-exec');

function setupStartEvents(previewWatch, done) {

    // Watch started
    previewWatch.once(consts.PREVIEW_START,
        function () {
            assert(true, 'Received expected Preview Start event.');
            done();
        }
    );

    // An error occurred
    previewWatch.on(consts.PREVIEW_ERROR,
        function() {
            assert(false, 'An error occurred while watching files.');
            done();
        }
    );
}

function setupModifiedEvents(previewWatch, done) {

    // A file change occurred
    previewWatch.once(consts.PREVIEW_RELOAD,
        function () {
            assert(true, 'Received expected Preview Reload event.');
            done();
        }
    );

    // An error occurred
    previewWatch.on(consts.PREVIEW_ERROR,
        function() {
            assert(false, 'An error occurred while watching files.');
            done();
        }
    );
}

function setupDeleteEvents(previewWatch, done) {

    // A file change occurred
    previewWatch.once(consts.PREVIEW_RELOAD,
        function () {
            assert(true, 'Received expected Preview Reload event.');
            done();
        }
    );

    // An error occurred
    previewWatch.on(consts.PREVIEW_ERROR,
        function() {
            assert(false, 'An error occurred while watching files.');
            done();
        }
    );
}

function modifyFile(filePath) {

  // Wait 1 second so Gaze doesn't miss this change
  setTimeout(
      function () {
        var now = (new Date()).getTime().toString();

        try {
            fs.writeFileSync(filePath, now);
        } catch (err) {
            assert(false, 'Could not modify a file.');
        }
      }, 1000);
}

function deleteFile(filePath) {

  // Wait 1 second so Gaze doesn't miss this change
  setTimeout(
      function () {
        try {
            fs.unlinkSync(filePath);
        } catch (err) {
            assert(false, 'Could not delete a file.');
        }
      }, 1000);
}

function invokePrepare() {
    cordovaSupport.prepare(consts.PREVIEW_ARG).then(
        function () {
            // Success
        },
        function () {
            assert(false, 'Cordova Prepare failed.');
        }
    );
}

describe('Test preview-watch',
    function () {
        var fixture;
        var destPath;
        var origCwd;

        before(
            function (done) {
                destPath = path.resolve(__dirname, 'fixtures');
                fixture = testFixtures('CordovaAndroidiOSWindows.zip', destPath);
                origCwd = process.cwd();

                exec.run('cordova', ['-v']).done(
                    function () {
                        fixture.clean();
                        process.chdir(path.resolve(destPath, 'CordovaAndroidiOSWindows'));
                        done();
                    },
                    function () {
                        assert(false, 'Cordova is not installed.');
                        done();
                    }
                );
            }
        );

        after(
            function () {
                fixture.delete();
                process.chdir(origCwd);
            }
        );

        describe('Test watch-and-copy',
            function () {
                var previewWatch;

                before(
                    function () {
                        previewWatch = new PreviewWatch('wc', path.resolve(),
                            ['android', 'ios', 'windows']);
                    }
                );

                after(
                    function () {
                        previewWatch.stopWatch();
                    }
                );

                it('Watch should be started',
                    function (done) {
                        setupStartEvents(previewWatch, done);
                        previewWatch.startWatch();
                    }
                );

                it('File should be modified',
                    function (done) {
                        var indexPath;

                        indexPath = path.resolve('www', 'index.html');
                        setupModifiedEvents(previewWatch, done);
                        modifyFile(indexPath);
                    }
                );

                it('File should be deleted',
                    function (done) {
                        var indexPath;

                        indexPath = path.resolve('www', 'index.html');
                        setupDeleteEvents(previewWatch, done);
                        deleteFile(indexPath);
                    }
                );
            }
        );

        describe('Test watch-and-prepare',
            function () {
                var previewWatch;

                before(
                    function () {
                        previewWatch = new PreviewWatch('wp', path.resolve(),
                            ['android', 'ios', 'windows']);
                    }
                );

                after(
                    function () {
                        previewWatch.stopWatch();
                    }
                );

                it('Watch should be started',
                    function (done) {
                        setupStartEvents(previewWatch, done);
                        previewWatch.startWatch();
                    }
                );

                it('File should be modified',
                    function (done) {
                        var indexPath;

                        indexPath = path.resolve('www', 'index.html');
                        setupModifiedEvents(previewWatch, done);
                        modifyFile(indexPath);
                    }
                );

                it('File should be deleted',
                    function (done) {
                        var indexPath;

                        indexPath = path.resolve('www', 'index.html');
                        setupDeleteEvents(previewWatch, done);
                        deleteFile(indexPath);
                    }
                );
            }
        );

        describe('Test watch-and-serve',
            function () {
                var previewWatch;

                before(
                    function () {
                        previewWatch = new PreviewWatch('s', path.resolve(),
                            ['android', 'ios', 'windows']);
                    }
                );

                after(
                    function () {
                        previewWatch.stopWatch();
                    }
                );

                it('Watch should be started',
                    function (done) {
                        setupStartEvents(previewWatch, done);
                        previewWatch.startWatch();
                    }
                );

                it('File should be modified',
                    function (done) {
                        var indexPath;

                        indexPath = path.resolve('www', 'index.html');
                        setupModifiedEvents(previewWatch, done);
                        modifyFile(indexPath);
                        invokePrepare();
                    }
                );

                it('File should be deleted',
                    function (done) {
                        var indexPath;

                        indexPath = path.resolve('www', 'index.html');
                        setupDeleteEvents(previewWatch, done);
                        deleteFile(indexPath);
                        invokePrepare();
                    }
                );
            }
        );
    }
);
