/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

/*global beforeEach, afterEach */
var assert = require('assert');
var path = require('path');
var mockery = require('mockery');
var Q = require('q');

global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');
global.MDO_CLI_FULL_VERSION = require('../../mfpdev-cli/package.json').version + '_mdo-server-support';
var getTestFixture = require('mdo-test-resources');

describe('unit tests for cli app push -', function () {

  this.timeout(10000);

  var inquirerRetVal = {};
  var fixturePath;
  var fixtureProject;
  var serverJSON;

  before(function () {
    mockery.enable({
      warnOnUnregistered: false
    });

    var inquirerMock = {
      prompt: function (messages, callback) {
        callback(inquirerRetVal);
      }
    };

    mockery.registerMock('inquirer', inquirerMock);

    var serverDetailsMock = {
            getServerDetails: function () {
                var d = Q.defer();
                d.resolve({
                    runtime: 'runtimez',
                    serverProfile: {
                        host: '1.2.3.4',
                        protocol: 'hzzp',
                        port: '1999'
                    },
                    setApplicationDescriptor: function (filePath, jsonPayload) {
                        var d = Q.defer();
                        d.resolve({
                            appInfo: jsonPayload.clientPlatform + ' was built',
                        });
                        serverJSON.push(jsonPayload);
                        return d.promise;
                    },
                    setRuntimeConfiguration: function () {
                        var d = Q.defer();
                        d.resolve();
                        return d.promise;
                    }
                });
                return d.promise;
            }
        };
        mockery.registerMock('mdo-server-support', serverDetailsMock);
  
  
  
  });

  beforeEach(function () {
      serverJSON = [];
      inquirerRetVal = {};
  });  
  after(function () {
    mockery.disable();
  });
  afterEach(function () {
      if (fixtureProject) {
         fixtureProject.delete();
      }
  });

  it('push command fails since not in proper project', function (done) {

    inquirerRetVal = {
      ipAddress: '127.0.0.1 (en0)'
    };

    require('../lib/push.js') . run (['localhost:3434'])
        .then(function () {
          assert(false, 'error should have been thrown');
        })
        .fail(function (error) {
          assert(
              error === 'NOPLATFORMSDETECTED',
              'Error: ' + error
          );
          done();
        })
        .catch(function (err) {
          done(err);
        });

  });

  it('push command succeeds', function (done) {

      fixturePath = path.resolve(__dirname, 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);
      fixtureProject.clean();

      var projPath = path.resolve(fixturePath, 'CordovaProject');
      process.chdir(projPath);

      inquirerRetVal = {
         ipAddress: '127.0.0.1 (en0)'
      };

      require('../lib/push.js') . run (['localhost:3434'])
      .then(function () {
         done();
      })
      .fail(function () {
        assert(false, 'error should not have been thrown');
        done();
      })
      .catch(function (err) {
        done(err);
      });

  });
  
  it('push command fails with runtime config feature and no runtime files', function (done) {

      fixturePath = path.resolve(__dirname, 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);
      fixtureProject.clean();

      var projPath = path.resolve(fixturePath, 'CordovaProject');
      process.chdir(projPath);

      inquirerRetVal = {
         ipAddress: '127.0.0.1 (en0)'
      };

      require('../lib/push.js') . run (['--runtime-config'])
      .then(function () {
         done();
      })
      .fail(function (error) {
        assert(
                  error === 'NORUNTIMECONFIGEXIST',
                  'Error: ' + error
              );
        done();
      })
      .catch(function (err) {
        done(err);
      });

  });
  
});
