/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach, afterEach */
var assert = require('assert');
var fs = require('fs');
var mockery = require('mockery');
var path = require('path');
var Q = require('q');
var rimraf = require('rimraf');
//private
var ConfigXMLAPI = require('mfp-config-xml').mfpConfigXMLAPI;
var errors = require('mdo-errors');
var getTestFixture = require('mdo-test-resources');

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

var verifyConfigXMLOriginalState = function (configXMLAPI) {
    assert.deepEqual(configXMLAPI.getMFPServerURL(), 'http://10.0.0.1:9080');
    assert.deepEqual(configXMLAPI.getMFPServerRuntime(), 'mfp');
    assert.deepEqual(configXMLAPI.getMFPPlatformVersion(), '');
    assert.deepEqual(configXMLAPI.getMFPDirectUpdateAuthKey(), '');
    assert.deepEqual(configXMLAPI.getMFPLanguagePrefs(), 'en');
    assert.deepEqual(configXMLAPI.getMFPClientCustomInit(), 'false');
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('android'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('ios'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('windows'), 0);
    var platforms = ['android', 'ios', 'windows8', 'windowsphone8', 'windows10'];
    //Check all other properties, if set do nothing, else set them

    for(var i = 0; i < platforms.length; i++) {
        assert.deepEqual(configXMLAPI.getMFPSDKChecksum(platforms[i]), 0);
        assert.deepEqual(configXMLAPI.getMFPTestWebResources(platforms[i]), 'false');
        assert.deepEqual(configXMLAPI.getMFPIgnoreFileExtensions(platforms[i]), 'png, jpg, jpeg, gif, mp4, mp3');
    }
};

var verifyConfigXML = function (configXMLAPI) {
    assert.deepEqual(configXMLAPI.getMFPServerURL(), 'hzzp:1.2.3.4:1999');
    assert.deepEqual(configXMLAPI.getMFPServerRuntime(), 'runtimez');
    assert.deepEqual(configXMLAPI.getMFPPlatformVersion(), '');
    assert.deepEqual(configXMLAPI.getMFPDirectUpdateAuthKey(), '');
    assert.deepEqual(configXMLAPI.getMFPLanguagePrefs(), 'en');
    assert.deepEqual(configXMLAPI.getMFPClientCustomInit(), 'false');
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('android'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('ios'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('windows'), 0);
    var platforms = ['android', 'ios', 'windows8', 'windowsphone8', 'windows10'];
    //Check all other properties, if set do nothing, else set them

    for (var i = 0; i < platforms.length; i++) {
        assert.deepEqual(configXMLAPI.getMFPSDKChecksum(platforms[i]), 0);
        assert.deepEqual(configXMLAPI.getMFPTestWebResources(platforms[i]), 'false');
        assert.deepEqual(configXMLAPI.getMFPIgnoreFileExtensions(platforms[i]), 'png, jpg, jpeg, gif, mp4, mp3');
    }
};

describe('app register', function () {
    var fixturePath,
        fixtureProject,
        registerApp,
        serverJSON;

    global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

    before(function () {
        mockery.enable({
            warnOnUnregistered: false,
            warnOnReplace: true
        });

        var serverDetailsMock = {
            getServerDetails: function () {
                var d = Q.defer();
                d.resolve({
                    runtime: 'runtimez',
                    serverProfile: {
                        host: '1.2.3.4',
                        protocol: 'hzzp',
                        port: '1999'
                    },
                    registerApplication: function (jsonPayload) {
                        var d = Q.defer();
                        d.resolve({
                            appInfo: jsonPayload.applicationKey.clientPlatform + ' was built',
                        });

                        serverJSON.push(jsonPayload);
                        return d.promise;
                    }
                });
                return d.promise;
            }
        };
        mockery.registerMock('mdo-server-support', serverDetailsMock);
        registerApp = require('../lib/index');
    });

    beforeEach(function () {
        serverJSON = [];
    });

    afterEach(function () {
        fixtureProject.delete();
    });
    after(function () {
        mockery.disable();
    });


    it('should be able to register Cordova app with android, ios, windows, browser platforms', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var propertyFilePath = path.resolve(projPath, 'config.xml');

        process.chdir(projPath);

        var configXML = new ConfigXMLAPI(propertyFilePath);
        verifyConfigXMLOriginalState(configXML);
        registerApp.run(['password'])
        .then(function () {
          configXML = new ConfigXMLAPI(propertyFilePath);
          verifyConfigXML(configXML);
          assert.deepEqual(serverJSON.length, 6, 'should have detected 6 apps');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'android', 'should have had android platform');
          assert.deepEqual(serverJSON[0].applicationKey.packageName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova packageName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[0].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[1].applicationKey.clientPlatform, 'web', 'should have had web platform');
          assert.deepEqual(serverJSON[1].applicationKey.applicationId, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova applicationId');
          assert.deepEqual(serverJSON[1].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[2].applicationKey.clientPlatform, 'ios', 'should have had ios platform');
          assert.deepEqual(serverJSON[2].applicationKey.bundleId, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova bundleId');
          assert.deepEqual(serverJSON[2].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[2].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[3].applicationKey.clientPlatform, 'windows', 'should have had windows platform');
          assert.deepEqual(serverJSON[3].applicationKey.assemblyName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova assemblyName');
          assert.deepEqual(serverJSON[3].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[3].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[4].applicationKey.clientPlatform, 'windows8', 'should have had windows8 platform');
          assert.deepEqual(serverJSON[4].applicationKey.assemblyName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova assemblyName');
          assert.deepEqual(serverJSON[4].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[4].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[5].applicationKey.clientPlatform, 'windowsphone8', 'should have had windowsphone8 platform');
          assert.deepEqual(serverJSON[5].applicationKey.assemblyName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova assemblyName');
          assert.deepEqual(serverJSON[5].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[5].displayName, 'HelloCordova', 'should have a displayName');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should be able to register Cordova app with android, ios, windows10, and browser using --windows flag', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var propertyFilePath = path.resolve(projPath, 'config.xml');

        process.chdir(projPath);

        var configXML = new ConfigXMLAPI(propertyFilePath);
        verifyConfigXMLOriginalState(configXML);
        registerApp.run(['password','--windows','windows'])
        .then(function () {
          configXML = new ConfigXMLAPI(propertyFilePath);
          verifyConfigXML(configXML);
          assert.deepEqual(serverJSON.length, 4, 'should have detected 4 apps');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'android', 'should have had android platform');
          assert.deepEqual(serverJSON[0].applicationKey.packageName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova packageName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[0].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[1].applicationKey.clientPlatform, 'web', 'should have had web platform');
          assert.deepEqual(serverJSON[1].applicationKey.applicationId, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova applicationId');
          assert.deepEqual(serverJSON[1].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[2].applicationKey.clientPlatform, 'ios', 'should have had ios platform');
          assert.deepEqual(serverJSON[2].applicationKey.bundleId, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova bundleId');
          assert.deepEqual(serverJSON[2].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[2].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[3].applicationKey.clientPlatform, 'windows', 'should have had windows platform');
          assert.deepEqual(serverJSON[3].applicationKey.assemblyName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova assemblyName');
          assert.deepEqual(serverJSON[3].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[3].displayName, 'HelloCordova', 'should have a displayName');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should be able to register Cordova app with android, ios, windows8, windowsphone8, browser platforms using --windows flag', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var propertyFilePath = path.resolve(projPath, 'config.xml');

        process.chdir(projPath);

        var configXML = new ConfigXMLAPI(propertyFilePath);
        verifyConfigXMLOriginalState(configXML);
        registerApp.run(['password','--windows','windows8,windowsphone8'])
        .then(function () {
          configXML = new ConfigXMLAPI(propertyFilePath);
          verifyConfigXML(configXML);
          assert.deepEqual(serverJSON.length, 5, 'should have detected 5 apps');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'android', 'should have had android platform');
          assert.deepEqual(serverJSON[0].applicationKey.packageName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova packageName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '0.0.1', 'should have had android platform');
          assert.deepEqual(serverJSON[0].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[1].applicationKey.clientPlatform, 'web', 'should have had web platform');
          assert.deepEqual(serverJSON[1].applicationKey.applicationId, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova applicationId');
          assert.deepEqual(serverJSON[1].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[2].applicationKey.clientPlatform, 'ios', 'should have had ios platform');
          assert.deepEqual(serverJSON[2].applicationKey.bundleId, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova bundleId');
          assert.deepEqual(serverJSON[2].applicationKey.version, '0.0.1', 'should have had android platform');
          assert.deepEqual(serverJSON[2].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[3].applicationKey.clientPlatform, 'windows8', 'should have had windows8 platform');
          assert.deepEqual(serverJSON[3].applicationKey.assemblyName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova assemblyName');
          assert.deepEqual(serverJSON[3].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[3].displayName, 'HelloCordova', 'should have a displayName');
          assert.deepEqual(serverJSON[4].applicationKey.clientPlatform, 'windowsphone8', 'should have had windowsphone8 platform');
          assert.deepEqual(serverJSON[4].applicationKey.assemblyName, 'io.cordova.hellocordova', 'should have had io.cordova.hellocordova assemblyName');
          assert.deepEqual(serverJSON[4].applicationKey.version, '0.0.1', 'should have had version 0.0.1');
          assert.deepEqual(serverJSON[4].displayName, 'HelloCordova', 'should have a displayName');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should register a native app Android', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('AndroidNativeProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'AndroidNativeProject');
        var propertyFilePath = path.resolve(projPath, 'app', 'src', 'main', 'assets', 'mfpclient.properties');

        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(propertyFilePath), 'mfpclient.properties should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.properties should exist');
          assert.deepEqual(serverJSON.length, 1, 'should have detected 1 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'android', 'should have had android platform');
          assert.deepEqual(serverJSON[0].applicationKey.packageName, 'test.ibm.com.testnativeproject', 'should have had valid packageName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '1.0', 'should have had valid version');
          assert.deepEqual(serverJSON[0].displayName, 'TestNativeProject', 'should have a displayName');
          cleanUpFile(propertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(propertyFilePath);
          done(err);
        });
    }),

    it('should register a native app IOS', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('IOSNativeProjectInfoPlistNoVars.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'IOSNativeProjectInfoPlistNoVars', 'TestNativeProject');
        var propertyFilePath = path.resolve(projPath, 'mfpclient.plist');

        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(propertyFilePath), 'mfpclient.properties should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.properties should exist');
          assert.deepEqual(serverJSON.length, 1, 'should have detected 1 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'ios', 'should have had ios platform');
          assert.deepEqual(serverJSON[0].applicationKey.bundleId, 'com.ibm.test', 'should have had valid packageName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '1.0', 'should have had valid version');
          assert.deepEqual(serverJSON[0].displayName, 'TestNativeProject', 'should have a displayName');
          cleanUpFile(propertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(propertyFilePath);
          done(err);
        });
    }),

    it('should register two Windows native app when it is Windows8', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('Windows8NativeProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'Windows8NativeProject', 'BlankSharedApp1');
        var windowsPropertyFilePath = path.resolve(projPath, 'BlankSharedApp1', 'BlankSharedApp1.Windows', 'strings', 'mfpclient.resw');
        var windowsPhonePropertyFilePath = path.resolve(projPath, 'BlankSharedApp1', 'BlankSharedApp1.WindowsPhone', 'strings', 'mfpclient.resw');
        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should not exist');
        assert.deepEqual(false, validateFileOrFolderExists(windowsPhonePropertyFilePath), 'WindowsPhone mfpclient.resw should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should exist');
          assert.deepEqual(true, validateFileOrFolderExists(windowsPhonePropertyFilePath), 'WindowsPhone mfpclient.resw should exist');
          assert.deepEqual(serverJSON.length, 2, 'should have detected 2 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'windows8', 'should have had windows8 platform');
          assert.deepEqual(serverJSON[0].applicationKey.assemblyName, 'BlankSharedApp1', 'should have had valid assemblyName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '1.0.0.0', 'should have had valid version');
          assert.deepEqual(serverJSON[0].displayName, 'BlankSharedApp1', 'should have a displayName');
          assert.deepEqual(serverJSON[1].applicationKey.clientPlatform, 'windowsphone8', 'should have had windowsphone8 platform');
          assert.deepEqual(serverJSON[1].applicationKey.assemblyName, 'BlankSharedApp1', 'should have had valid assemblyName');
          assert.deepEqual(serverJSON[1].applicationKey.version, '1.0.0.0', 'should have had valid version');
          assert.deepEqual(serverJSON[1].displayName, 'BlankSharedApp1', 'should have a displayName');
          cleanUpFile(windowsPropertyFilePath);
          cleanUpFile(windowsPhonePropertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(windowsPropertyFilePath);
          cleanUpFile(windowsPhonePropertyFilePath);
          done(err);
        });
    }),

    it('should register two Windows native app when it is Windows8 alternate folder names', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('Windows8AltNativeProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'Windows8AltNativeProject', 'Compression');
        var windowsPropertyFilePath = path.resolve(projPath, 'Windows', 'strings', 'mfpclient.resw');
        var windowsPhonePropertyFilePath = path.resolve(projPath, 'WindowsPhone', 'strings', 'mfpclient.resw');
        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should not exist');
        assert.deepEqual(false, validateFileOrFolderExists(windowsPhonePropertyFilePath), 'WindowsPhone mfpclient.resw should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should exist');
          assert.deepEqual(true, validateFileOrFolderExists(windowsPhonePropertyFilePath), 'WindowsPhone mfpclient.resw should exist');
          assert.deepEqual(serverJSON.length, 2, 'should have detected 2 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'windows8', 'should have had windows8 platform');
          assert.deepEqual(serverJSON[1].applicationKey.clientPlatform, 'windowsphone8', 'should have had windowsphone8 platform');
          cleanUpFile(windowsPropertyFilePath);
          cleanUpFile(windowsPhonePropertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(windowsPropertyFilePath);
          cleanUpFile(windowsPhonePropertyFilePath);
          done(err);
        });
    }),

    it('should register one Windows native app when it is a Windows8 Desktop Only project', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('Windows8DesktopOnly.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'Windows8DesktopOnly');
        var windowsPropertyFilePath = path.resolve(projPath, 'Windows8DesktopOnly', 'strings', 'mfpclient.resw');
        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should exist');
          assert.deepEqual(serverJSON.length, 1, 'should have detected 1 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'windows8', 'should have had windows8 platform');
          assert.deepEqual(serverJSON[0].applicationKey.assemblyName, 'DesktopOnly', 'should have had valid assemblyName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '1.0.0.0', 'should have had valid version');
          assert.deepEqual(serverJSON[0].displayName, 'DesktopOnly', 'should have a displayName');
          cleanUpFile(windowsPropertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(windowsPropertyFilePath);
          done(err);
        });
    }),

    it('should register one Windows native app when it is a Windows8 Phone Only project', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('Windows8PhoneOnly.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'Windows8PhoneOnly');
        var windowsPropertyFilePath = path.resolve(projPath, 'Windows8PhoneOnly', 'strings', 'mfpclient.resw');
        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(windowsPropertyFilePath), 'Windows mfpclient.resw should exist');
          assert.deepEqual(serverJSON.length, 1, 'should have detected 1 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'windowsphone8', 'should have had windows8 platform');
          assert.deepEqual(serverJSON[0].applicationKey.assemblyName, 'PhoneOnly', 'should have had valid assemblyName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '1.0.0.0', 'should have had valid version');
          assert.deepEqual(serverJSON[0].displayName, 'PhoneOnly', 'should have a displayName');
          cleanUpFile(windowsPropertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(windowsPropertyFilePath);
          done(err);
        });
    }),

    it('should register only one Windows native app when it is Windows10', function (done) {
        fixturePath = path.resolve(__dirname, 'fixtures');
        fixtureProject = getTestFixture('Windows10NativeProject.zip', fixturePath);

        fixtureProject.clean();
        var projPath = path.resolve(fixturePath, 'Windows10NativeProject', 'App2');
        var propertyFilePath = path.resolve(projPath, 'strings', 'mfpclient.resw');

        process.chdir(projPath);

        assert.deepEqual(false, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should not exist');
        registerApp.run(['password'])
        .then(function () {
          assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should exist');
          assert.deepEqual(serverJSON.length, 1, 'should have detected 1 app');
          assert.deepEqual(serverJSON[0].applicationKey.clientPlatform, 'windows', 'should have had windows platform');
          assert.deepEqual(serverJSON[0].applicationKey.assemblyName, 'App2', 'should have had valid assemblyName');
          assert.deepEqual(serverJSON[0].applicationKey.version, '1.0.0.0', 'should have had valid version');
          assert.deepEqual(serverJSON[0].displayName, 'App2', 'should have a displayName');
          cleanUpFile(propertyFilePath);
          done();
        }).catch(function (err) {
          cleanUpFile(propertyFilePath);
          done(err);
        });
    }),

    it('should not register a native app when no platforms are detected', function (done) {
        var projPath = path.resolve(__dirname);
        process.chdir(projPath);

        var noPlatformsErr = errors.getError('NOPLATFORMSDETECTED');

        registerApp.run(['password'])
        .then(function () {
          assert(false, 'should not have detected any apps and should have failed');
        },function (err) {
          assert.deepEqual(serverJSON.length, 0, 'should not have detected any apps');
          assert.deepEqual(err.description, noPlatformsErr.description);
          assert.deepEqual(err.errno, noPlatformsErr.errno);
          done();
        }).catch(function (err) {
          done(err);
        });
    });
});

function cleanUpFile(filepath) {
  rimraf.sync(filepath);
}
