/*
	Licensed Material - Property of IBM

	(C) Copyright 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach, afterEach */
var assert = require('assert');
var mockery = require('mockery');
var path = require('path');
var Q = require('q');
//private
var getTestFixture = require('mdo-test-resources');
var cordovaSupport = require('mdo-cordova-support');

describe('app encrypt, encrypt passes', function () {
  this.timeout(10000);
    var projPath,
        fixturePath,
        fixtureProject,
        nativeFixtureProject,
        encryptedCordovaProject,
        encrypt,
        encryptPassed,
        encryptFunction,
        encryptPlatforms;

    global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

    before(function () {
      encryptPlatforms = [];
      encryptPassed = false;
      //Set up the intial functions we are mocking
      encryptFunction = encryptPass;

      fixturePath = path.resolve(__dirname, 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);
      projPath = path.resolve(fixturePath, 'CordovaProject');

      nativeFixtureProject = getTestFixture('IOSNativeProject.zip', fixturePath);
      encryptedCordovaProject = getTestFixture('EncryptedCordovaProject.zip', fixturePath);
    });

    beforeEach(function () {
        mockery.enable({
            useCleanCache: true,
            warnOnUnregistered: false,
            warnOnReplace: false
        });

        var cordovaSupportMock = {
          encryptWWWFolder: encryptFunction,
          detect: cordovaSupport.detect,
          parse: cordovaSupport.parse
        };
        mockery.registerMock('mdo-cordova-support', cordovaSupportMock);
        encrypt = require('../lib/index');
    });

    afterEach(function () {
        encryptPlatforms = [];
        encryptPassed = false;
    });

    after(function () {
      fixtureProject.delete();
      nativeFixtureProject.delete();
      encryptedCordovaProject.delete();
      mockery.disable();
    });


    it('should perform encryption for both android and ios platforms, windows is excluded', function (done) {
        fixtureProject.clean();
        var androidEncryptPath = path.resolve(projPath, 'platforms', 'android', 'assets', 'www');
        var iosEncryptPath = path.resolve(projPath, 'platforms', 'ios', 'www');

        assert.deepEqual(encryptPassed, false, 'encryptPassed should be false at this point');
        assert.deepEqual(encryptPlatforms.length, 0, 'encryptPlatforms should be 0');
        process.chdir(projPath);

        encrypt.run([])
        .then(function () {
          assert.deepEqual(encryptPassed, true, 'encrypt script should have been called');
          assert.deepEqual(encryptPlatforms.length, 2, 'should have encrypted both android and ios');
          var androidEncryptArgs = ['--path', androidEncryptPath, '--chunk'];
          assert.deepEqual(encryptPlatforms[0].join(), androidEncryptArgs.join(), 'should have sent correct args to encrypt android');
          var iosEncryptArgs = ['--path', iosEncryptPath, ''];
          assert.deepEqual(encryptPlatforms[1].join(), iosEncryptArgs.join(), 'shoudl have sent correct args to encrypt ios');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should perform encrypt for ios platform and leave android alone', function (done) {
        fixtureProject.clean();
        var iosEncryptPath = path.resolve(projPath, 'platforms', 'ios', 'www');

        var iosPath = path.resolve(projPath, 'platforms', 'ios');
        assert.deepEqual(encryptPassed, false, 'encryptPassed should be false at this point');
        assert.deepEqual(encryptPlatforms.length, 0, 'encryptPlatforms should be 0');
        process.chdir(iosPath);

        encrypt.run([])
        .then(function () {
          assert.deepEqual(encryptPassed, true, 'encrypt script should have been called');
          assert.deepEqual(encryptPlatforms.length, 1, 'should have encrypted ios');
          var iosEncryptArgs = ['--path', iosEncryptPath, ''];
          assert.deepEqual(encryptPlatforms[0].join(), iosEncryptArgs.join(), 'shoudl have sent correct args to encrypt ios');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should only perform encrypt on just android and leave ios alone', function (done) {
        fixtureProject.clean();
        var androidEncryptPath = path.resolve(projPath, 'platforms', 'android', 'assets', 'www');

        var androidPath = path.resolve(projPath, 'platforms', 'android');
        assert.deepEqual(encryptPassed, false, 'encryptPassed should be false at this point');
        assert.deepEqual(encryptPlatforms.length, 0, 'encryptPlatforms should be 0');
        process.chdir(androidPath);

        encrypt.run([])
        .then(function () {
          assert.deepEqual(encryptPassed, true, 'encrypt script should have been called');
          assert.deepEqual(encryptPlatforms.length, 1, 'should have encrypted android');
          var androidEncryptArgs = ['--path', androidEncryptPath, '--chunk'];
          assert.deepEqual(encryptPlatforms[0].join(), androidEncryptArgs.join(), 'should have sent correct args to encrypt android');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not perform encrypt if already encrypted', function (done) {
        encryptedCordovaProject.clean();
        var encryptedCordovaProjPath = path.resolve(fixturePath, 'EncryptedCordovaProject');

        assert.deepEqual(encryptPassed, false, 'encryptPassed should be false at this point');
        assert.deepEqual(encryptPlatforms.length, 0, 'encryptPlatforms should be 0');
        process.chdir(encryptedCordovaProjPath);

        encrypt.run([])
        .then(function () {
          assert.deepEqual(encryptPassed, false, 'encryptPassed should not have been called');
          assert.deepEqual(encryptPlatforms.length, 0, 'encryptPlatforms length should be 0');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not perform encrypt when in native project', function (done) {
        nativeFixtureProject.clean();
        var nativeProjPath = path.resolve(fixturePath, 'IOSNativeProject');

        process.chdir(nativeProjPath);

        encrypt.run([])
        .then(function () {
          assert(false, 'should not have been successful in native project');
        },function (err) {
          assert.notDeepEqual(err, undefined, 'Error should not be undefined');
          assert.deepEqual(err.errno, 188, 'Should have reported back an error no. 171');
          assert.deepEqual(err.code, 'MUSTBECORDOVAPROJECT', 'Should have got back error MUSTBECORDOVAPROJECT');
          nativeFixtureProject.delete();
          done();
        })
        .catch(function (err) {
          nativeFixtureProject.delete();
          done(err);
        });
    }),

    it('should not perform encrypt if no platforms are detected', function (done) {
        process.chdir(__dirname);

        encrypt.run([])
        .then(function () {
          assert(false, 'should not have entered success funciton');
        },function (err) {
          assert.notDeepEqual(err, undefined, 'Error should not be undefined');
          assert.deepEqual(err.errno, 145, 'Should have reported back an error no. 145');
          assert.deepEqual(err.code, 'NOPLATFORMSDETECTED', 'Should have got back error NOPLATFORMSDETECTED');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    });

    function encryptPass (configXMLDir, platform) {
      encryptPassed = true;
      encryptPlatforms.push(platform);
      return createReturnPromise();
    }

    function createReturnPromise() {
      var d = Q.defer();
      d.resolve();

      return d.promise;
    }
});
