/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach, afterEach */
var assert = require('assert');
var fs = require('fs');
var mockery = require('mockery');
var path = require('path');
var rimraf = require('rimraf');
var Q = require('q');
//private
var getTestFixture = require('mdo-test-resources');
var cordovaSupport = require('mdo-cordova-support');

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

describe('app webupdate, scripts prepare pass, checksum fails', function () {
  this.timeout(10000);
    var fixturePath,
        fixtureProject,
        webupdate,
        serverArgs,
        preparePassed,
        checksumPassed,
        checksumPlatforms,
        preparePlatforms,
        applicationsOnServer,
        projPath,
        returnCode;


    global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

    before(function () {
      serverArgs = [];
      preparePlatforms = [];
      checksumPlatforms = [];
      preparePassed = false;
      checksumPassed = false;
      //Set up the intial functions we are mocking
      applicationsOnServer = {
        items: [
          {
            'name': 'io.cordova.hellocordova',
            'envs': [{'environment':'android'},{'environment':'ios'}]
          }]
      };

      fixturePath = path.resolve(__dirname, 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);
      fixtureProject.clean();
      projPath = path.resolve(fixturePath, 'CordovaProject');
    });

    beforeEach(function () {
        removeMobileFirstDir(projPath);

        mockery.enable({
            useCleanCache: true,
            warnOnUnregistered: false,
            warnOnReplace: false
        });

        var serverDetailsMock = {
            getServerDetails: function () {
                var d = Q.defer();
                d.resolve({
                    runtime: 'runtimez',
                    serverProfile: {
                        host: '1.2.3.4',
                        protocol: 'hzzp',
                        port: '1999'
                    },
                    deployWebupdate: function (jsonPayload, file) {
                        var d = Q.defer();
                        d.resolve({});

                        var args = {
                          json: jsonPayload,
                          file: file
                        };
                        serverArgs.push(args);

                        return d.promise;
                    },
                    getApplications: function () {
                      var d = Q.defer();
                      d.resolve(applicationsOnServer);

                      return d.promise;
                    }
                });
                return d.promise;
            }
        };

        var cordovaSupportMock = {
          prepare: preparePass,
          generateChecksum: failureFunction,
          detect: cordovaSupport.detect,
          parse: cordovaSupport.parse
        };
        mockery.registerMock('mdo-server-support', serverDetailsMock);
        mockery.registerMock('mdo-cordova-support', cordovaSupportMock);
        webupdate = require('../lib/index');
    });

    afterEach(function () {
        serverArgs = [];
        preparePlatforms = [];
        checksumPlatforms = [];
        preparePassed = false;
        checksumPassed = false;
    });
    after(function () {
      fixtureProject.delete();
      mockery.disable();
    });


    it('should not perform webupdate as checksum function should fail with -2 exit code', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        //Set return code to be -2 to simulate platform version error in checksum code
        returnCode = -2;

        webupdate.run([])
        .then(function () {
          assert(false, 'should not have entered success function');
        },function (err) {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
          assert.notDeepEqual(err, undefined, 'error should not be undefined');
          assert.deepEqual(err.errno, 254, 'error number should be 254');
          assert.deepEqual(err.code, 'INVALIDPLATFORMVERSION', 'error code should be INVALIDPLATFORMVERSION');
          assert.deepEqual(preparePassed, true, 'prepare should have executed');
          assert.deepEqual(checksumPassed, false, 'checksum should have failed');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate as checksum function should fail with 254 exit code', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        //Set return code to be 254 to simulate platform version error in checksum code
        returnCode = 254;

        webupdate.run([])
        .then(function () {
          assert(false, 'should not have entered success function');
        },function (err) {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
          assert.notDeepEqual(err, undefined, 'error should not be undefined');
          assert.deepEqual(err.errno, 254, 'error number should be 254');
          assert.deepEqual(err.code, 'INVALIDPLATFORMVERSION', 'error code should be INVALIDPLATFORMVERSION');
          assert.deepEqual(preparePassed, true, 'prepare should have executed');
          assert.deepEqual(checksumPassed, false, 'checksum should have failed');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate as checksum function should fail with non-zero exit code', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        //Set return code to be -2 to simulate platform version error in checksum code
        returnCode = -1;

        webupdate.run([])
        .then(function () {
          assert(false, 'should not have entered success function');
        },function (err) {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
          assert.notDeepEqual(err, undefined, 'error should not be undefined');
          assert.deepEqual(err.errno, 255, 'error number should be 255');
          assert.deepEqual(err.code, 'ERRORDURINGEXTERNALSCRIPT', 'error code should be ERRORDURINGEXTERNALSCRIPT');
          assert.deepEqual(preparePassed, true, 'prepare should have executed');
          assert.deepEqual(checksumPassed, false, 'checksum should have failed');
          done();
        }).catch(function (err) {
          done(err);
        });
    });

    function preparePass (platform) {
      preparePassed = true;
      preparePlatforms.push(platform);
      return createReturnPromise();
    }

    function failureFunction () {
      return createReturnPromise(true, {
        code: returnCode
      });
    }

    function createReturnPromise(reject, errObj) {
      var d = Q.defer();
      if(reject) {
        d.reject(errObj);
      } else {
        d.resolve();
      }

      return d.promise;
    }
});

function removeMobileFirstDir(projectPath) {
  var mobilefirstDir = path.resolve(projectPath, 'mobilefirst');
  rimraf.sync(mobilefirstDir);
}
