/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach, afterEach */
var admZip = require('adm-zip');
var assert = require('assert');
var fs = require('fs');
var mockery = require('mockery');
var path = require('path');
var rimraf = require('rimraf');
var Q = require('q');
//private
var getTestFixture = require('mdo-test-resources');
var cordovaSupport = require('mdo-cordova-support');

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

describe('app webupdate, scripts prepare,checksum pass', function () {
  this.timeout(10000);
    var fixturePath,
        fixtureProject,
        nativeFixtureProject,
        webupdate,
        serverArgs,
        preparePassed,
        checksumPassed,
        checksumFunction,
        prepareFunction,
        checksumPlatforms,
        preparePlatforms,
        applicationsOnServer,
        projPath;


    global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

    before(function () {
      serverArgs = [];
      preparePlatforms = [];
      checksumPlatforms = [];
      preparePassed = false;
      checksumPassed = false;
      //Set up the intial functions we are mocking
      prepareFunction = preparePass;
      checksumFunction = checksumPass;
      applicationsOnServer = {
        items: [
          {
            'name': 'io.cordova.hellocordova',
            'envs': [{'environment':'android'},{'environment':'ios'}]
          }]
      };

      fixturePath = path.resolve(__dirname, 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);
      fixtureProject.clean();
      projPath = path.resolve(fixturePath, 'CordovaProject');

      nativeFixtureProject = getTestFixture('IOSNativeProject.zip', fixturePath);
    });

    beforeEach(function () {
        removeMobileFirstDir(projPath);

        mockery.enable({
            useCleanCache: true,
            warnOnUnregistered: false,
            warnOnReplace: false
        });

        var serverDetailsMock = {
            getServerDetails: function () {
                var d = Q.defer();
                d.resolve({
                    runtime: 'runtimez',
                    serverProfile: {
                        host: '1.2.3.4',
                        protocol: 'hzzp',
                        port: '1999'
                    },
                    deployWebupdate: function (jsonPayload, file) {
                        var d = Q.defer();
                        d.resolve({});

                        var args = {
                          json: jsonPayload,
                          file: file
                        };
                        serverArgs.push(args);

                        return d.promise;
                    },
                    getApplications: function () {
                      var d = Q.defer();
                      d.resolve(applicationsOnServer);

                      return d.promise;
                    }
                });
                return d.promise;
            }
        };

        var cordovaSupportMock = {
          prepare: prepareFunction,
          generateChecksum: checksumFunction,
          detect: cordovaSupport.detect,
          parse: cordovaSupport.parse
        };
        mockery.registerMock('mdo-server-support', serverDetailsMock);
        mockery.registerMock('mdo-cordova-support', cordovaSupportMock);
        webupdate = require('../lib/index');
    });

    afterEach(function () {
        serverArgs = [];
        preparePlatforms = [];
        checksumPlatforms = [];
        preparePassed = false;
        checksumPassed = false;
    });
    after(function () {
      fixtureProject.delete();
      mockery.disable();
    });


    it('should perform webupdate for both android and ios platforms, windows is excluded', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run([])
        .then(function () {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), true, 'mobilefirst dir should have been created');
          assert.deepEqual(preparePassed, true, 'cordova prepare should pass on a successful run');
          assert.deepEqual(preparePlatforms.length, 2, 'prepare should have been called for android and ios');
          assert.deepEqual(preparePlatforms[0], 'android', 'should have prepared android');
          assert.deepEqual(preparePlatforms[1], 'ios', 'should have prepared ios');
          assert.deepEqual(checksumPassed, true, 'checksum script should have been called');
          assert.deepEqual(checksumPlatforms.length, 2, 'checksum script should have been called for both platforms');
          var androidChecksumArgs = ['--path',projPath,'--platforms','android'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[0].join(), androidChecksumArgs.join(), 'should have sent the correct args to checksumed android');
          var iosChecksumArgs = ['--path',projPath,'--platforms','ios'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[1].join(), iosChecksumArgs.join(), 'should have sent the correct args to checksumed ios');
          assert.deepEqual(serverArgs.length, 2, 'server should have deployed 2 files');
          assert.deepEqual(serverArgs[0].json.clientPlatform, 'android', 'should have deployed android');
          var androidFilename = serverArgs[0].json.packageName + '-' + serverArgs[0].json.clientPlatform + '-' + serverArgs[0].json.version + '.zip';
          assert.deepEqual(path.basename(serverArgs[0].file), androidFilename);
          assert.deepEqual(serverArgs[1].json.clientPlatform, 'ios', 'should have deployed ios');
          var iosFilename = serverArgs[1].json.bundleId + '-' + serverArgs[1].json.clientPlatform + '-' + serverArgs[1].json.version + '.zip';
          assert.deepEqual(path.basename(serverArgs[1].file), iosFilename);

          //Verify files were created in mobilefirst directory
          var androidFilePath = path.resolve(mobilefirstPath, androidFilename);
          assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file should have been created in mobilefirst dir');
          var iosFilePath = path.resolve(mobilefirstPath, iosFilename);
          assert.deepEqual(validateFileOrFolderExists(iosFilePath), true, 'ios file should have been created in mobilefirst dir');

          assert.deepEqual(verifyContentOfResourceFile(androidFilePath, 'android'), true, 'all needed files should be in android zip');
          assert.deepEqual(verifyContentOfResourceFile(iosFilePath, 'ios'), true, 'all needed files should be in ios zip');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should perform webupdate for ios platforms', function (done) {
        var iosPath = path.resolve(projPath, 'platforms', 'ios');
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(iosPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run([])
        .then(function () {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), true, 'mobilefirst dir should have been created');
          assert.deepEqual(preparePassed, true, 'cordova prepare should pass on a successful run');
          assert.deepEqual(preparePlatforms.length, 1, 'prepare should have been called for ios');
          assert.deepEqual(preparePlatforms[0], 'ios', 'should have prepared ios');
          assert.deepEqual(checksumPassed, true, 'checksum script should have been called');
          assert.deepEqual(checksumPlatforms.length, 1, 'checksum script should have been called for ios');
          var iosChecksumArgs = ['--path',projPath,'--platforms','ios'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[0].join(), iosChecksumArgs.join(), 'should have sent the correct args to checksumed ios');
          assert.deepEqual(serverArgs.length, 1, 'server should have deployed ios files');
          assert.deepEqual(serverArgs[0].json.clientPlatform, 'ios', 'should have deployed ios');
          var iosFilename = serverArgs[0].json.bundleId + '-' + serverArgs[0].json.clientPlatform + '-' + serverArgs[0].json.version + '.zip';
          assert.deepEqual(path.basename(serverArgs[0].file), iosFilename);

          //Verify files were created in mobilefirst directory
          var iosFilePath = path.resolve(mobilefirstPath, iosFilename);
          assert.deepEqual(validateFileOrFolderExists(iosFilePath), true, 'ios file should have been created in mobilefirst dir');

          assert.deepEqual(verifyContentOfResourceFile(iosFilePath, 'ios'), true, 'all needed files should be in ios zip');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should only perform webupdate on just android when in android platform', function (done) {
        var androidPath = path.resolve(projPath, 'platforms', 'android');
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(androidPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run([])
        .then(function () {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), true, 'mobilefirst dir should have been created');
          assert.deepEqual(preparePassed, true, 'cordova prepare should pass on a successful run');
          assert.deepEqual(preparePlatforms.length, 1, 'prepare should have been called for android');
          assert.deepEqual(preparePlatforms[0], 'android', 'should have prepared android');
          assert.deepEqual(checksumPassed, true, 'checksum script should have been called');
          assert.deepEqual(checksumPlatforms.length, 1, 'checksum script should have been called for android platforms');
          var androidChecksumArgs = ['--path',projPath,'--platforms','android'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[0].join(), androidChecksumArgs.join(), 'should have sent the correct args to checksumed android');
          assert.deepEqual(serverArgs.length, 1, 'server should have deployed 1 files');
          assert.deepEqual(serverArgs[0].json.clientPlatform, 'android', 'should have deployed android');
          var androidFilename = serverArgs[0].json.packageName + '-' + serverArgs[0].json.clientPlatform + '-' + serverArgs[0].json.version + '.zip';
          assert.deepEqual(path.basename(serverArgs[0].file), androidFilename);

          //Verify files were created in mobilefirst directory
          var androidFilePath = path.resolve(mobilefirstPath, androidFilename);
          assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file should have been created in mobilefirst dir');

          assert.deepEqual(verifyContentOfResourceFile(androidFilePath, 'android'), true, 'all needed files should be in android zip');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should perform webupdate for both android and ios platforms with --build, windows is excluded', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run(['--build'])
        .then(function () {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), true, 'mobilefirst dir should have been created');
          assert.deepEqual(preparePassed, true, 'cordova prepare should pass on a successful run');
          assert.deepEqual(preparePlatforms.length, 2, 'prepare should have been called for android and ios');
          assert.deepEqual(preparePlatforms[0], 'android', 'should have prepared android');
          assert.deepEqual(preparePlatforms[1], 'ios', 'should have prepared ios');
          assert.deepEqual(checksumPassed, true, 'checksum script should have been called');
          assert.deepEqual(checksumPlatforms.length, 2, 'checksum script should have been called for both platforms');
          var androidChecksumArgs = ['--path',projPath,'--platforms','android'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[0].join(), androidChecksumArgs.join(), 'should have sent the correct args to checksumed android');
          var iosChecksumArgs = ['--path',projPath,'--platforms','ios'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[1].join(), iosChecksumArgs.join(), 'should have sent the correct args to checksumed ios');
          assert.deepEqual(serverArgs.length, 0, 'server should not have deployed 2 files');
          var androidFilename = 'io.cordova.hellocordova-android-0.0.1.zip';
          var iosFilename = 'io.cordova.hellocordova-ios-0.0.1.zip';

          //Verify files were created in mobilefirst directory
          var androidFilePath = path.resolve(mobilefirstPath, androidFilename);
          assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file should have been created in mobilefirst dir');
          var iosFilePath = path.resolve(mobilefirstPath, iosFilename);
          assert.deepEqual(validateFileOrFolderExists(iosFilePath), true, 'ios file should have been created in mobilefirst dir');

          assert.deepEqual(verifyContentOfResourceFile(androidFilePath, 'android'), true, 'all needed files should be in android zip');
          assert.deepEqual(verifyContentOfResourceFile(iosFilePath, 'ios'), true, 'all needed files should be in ios zip');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should perform webupdate for ios platforms with --build', function (done) {
        var iosPath = path.resolve(projPath, 'platforms', 'ios');
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(iosPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run(['--build'])
        .then(function () {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), true, 'mobilefirst dir should have been created');
          assert.deepEqual(preparePassed, true, 'cordova prepare should pass on a successful run');
          assert.deepEqual(preparePlatforms.length, 1, 'prepare should have been called for ios');
          assert.deepEqual(preparePlatforms[0], 'ios', 'should have prepared ios');
          assert.deepEqual(checksumPassed, true, 'checksum script should have been called');
          assert.deepEqual(checksumPlatforms.length, 1, 'checksum script should have been called for ios');
          var iosChecksumArgs = ['--path',projPath,'--platforms','ios'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[0].join(), iosChecksumArgs.join(), 'should have sent the correct args to checksumed ios');
          assert.deepEqual(serverArgs.length, 0, 'server should not have deployed ios files');
          var iosFilename = 'io.cordova.hellocordova-ios-0.0.1.zip';

          //Verify files were created in mobilefirst directory
          var iosFilePath = path.resolve(mobilefirstPath, iosFilename);
          assert.deepEqual(validateFileOrFolderExists(iosFilePath), true, 'ios file should have been created in mobilefirst dir');

          assert.deepEqual(verifyContentOfResourceFile(iosFilePath, 'ios'), true, 'all needed files should be in ios zip');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should only perform webupdate on just android when in android platform with --build', function (done) {
        var androidPath = path.resolve(projPath, 'platforms', 'android');
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(androidPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run(['--build'])
        .then(function () {
          assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), true, 'mobilefirst dir should have been created');
          assert.deepEqual(preparePassed, true, 'cordova prepare should pass on a successful run');
          assert.deepEqual(preparePlatforms.length, 1, 'prepare should have been called for android');
          assert.deepEqual(preparePlatforms[0], 'android', 'should have prepared android');
          assert.deepEqual(checksumPassed, true, 'checksum script should have been called');
          assert.deepEqual(checksumPlatforms.length, 1, 'checksum script should have been called for android platforms');
          var androidChecksumArgs = ['--path',projPath,'--platforms','android'];
          //Compare the joined array strings to check that correct args were passed to checksum
          assert.deepEqual(checksumPlatforms[0].join(), androidChecksumArgs.join(), 'should have sent the correct args to checksumed android');
          assert.deepEqual(serverArgs.length, 0, 'server should not have deployed android file');
          var androidFilename = 'io.cordova.hellocordova-android-0.0.1.zip';

          //Verify files were created in mobilefirst directory
          var androidFilePath = path.resolve(mobilefirstPath, androidFilename);
          assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file should have been created in mobilefirst dir');
          assert.deepEqual(verifyContentOfResourceFile(androidFilePath, 'android'), true, 'all needed files should be in android zip');

          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should only perform webupdate on just android when in android platform with --file', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var androidFilePath = path.resolve(mobilefirstPath, 'io.cordova.hellocordova-android-0.0.1.zip');
        //Create a resource file under the mobilefirst dir
        fs.mkdirSync(mobilefirstPath);
        fs.writeFileSync(androidFilePath, 'androidFile');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file we just created should exist');
        webupdate.run(['--file', androidFilePath])
        .then(function () {
          assert.deepEqual(preparePassed, false, 'cordova prepare should not have been called');
          assert.deepEqual(preparePlatforms.length, 0, 'prepare should not have been called for android and ios');
          assert.deepEqual(checksumPassed, false, 'checksum script should not have been called');
          assert.deepEqual(checksumPlatforms.length, 0, 'checksum script should not have been called for both platforms');
          assert.deepEqual(serverArgs.length, 1, 'server should have deployed 1 files');
          assert.deepEqual(serverArgs[0].json.clientPlatform, 'android', 'should have deployed android');
          var androidFilename = serverArgs[0].json.packageName + '-' + serverArgs[0].json.clientPlatform + '-' + serverArgs[0].json.version + '.zip';
          assert.deepEqual(path.basename(serverArgs[0].file), androidFilename);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should only perform webupdate on just android when in ios platform with --file', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var iosFilePath = path.resolve(mobilefirstPath, 'io.cordova.hellocordova-ios-0.0.1.zip');
        //Create a resource file under the mobilefirst dir
        fs.mkdirSync(mobilefirstPath);
        fs.writeFileSync(iosFilePath, 'iosFile');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(iosFilePath), true, 'ios file we just created should exist');
        webupdate.run(['--file', iosFilePath])
        .then(function () {
          assert.deepEqual(preparePassed, false, 'cordova prepare should not have been called');
          assert.deepEqual(preparePlatforms.length, 0, 'prepare should not have been called for android and ios');
          assert.deepEqual(checksumPassed, false, 'checksum script should not have been called');
          assert.deepEqual(checksumPlatforms.length, 0, 'checksum script should not have been called for both platforms');
          assert.deepEqual(serverArgs.length, 1, 'server should have deployed 1 files');
          assert.deepEqual(serverArgs[0].json.clientPlatform, 'ios', 'should have deployed android');
          var iosFilename = serverArgs[0].json.bundleId + '-' + serverArgs[0].json.clientPlatform + '-' + serverArgs[0].json.version + '.zip';
          assert.deepEqual(path.basename(serverArgs[0].file), iosFilename);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate when app is not registered', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var currentRegisteredApps = applicationsOnServer;

        applicationsOnServer = {
          items: [
            {
              'name': 'io.cordova.somethingelse',
              'envs': [{'environment':'android'},{'environment':'ios'}]
            }]
        };

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run([])
        .then(function () {
          assert(false, 'it should not have entered the success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'error should not be undefined');
          assert.deepEqual(err.errno, 178, 'error number should be ' );
          assert.deepEqual(err.code, 'APPNOTREGISTEREDBEFOREDEPLOYMENT', 'error code should be APPNOTREGISTEREDBEFOREDEPLOYMENT');
          applicationsOnServer = currentRegisteredApps;
          done();
        }).catch(function (err) {
          applicationsOnServer = currentRegisteredApps;
          done(err);
        });
    }),

    it('should not perform webupdate with windows file using --file', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var androidFilePath = path.resolve(mobilefirstPath, 'io.cordova.hellocordova-windows-0.0.1.zip');
        //Create a resource file under the mobilefirst dir
        fs.mkdirSync(mobilefirstPath);
        fs.writeFileSync(androidFilePath, 'androidFile');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file we just created should exist');
        webupdate.run(['--file', androidFilePath])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 172, 'Error number should be 172');
          assert.deepEqual(err.code, 'WINDOWSNOTSUPPORTEDWEBUPDATE', 'Error code should be WINDOWSNOTSUPPORTEDWEBUPDATE');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate with invalid platform name in filename, --file', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var androidFilePath = path.resolve(mobilefirstPath, 'io.cordova.hellocordova-invalidPlatform-0.0.1.zip');
        //Create a resource file under the mobilefirst dir
        fs.mkdirSync(mobilefirstPath);
        fs.writeFileSync(androidFilePath, 'androidFile');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file we just created should exist');
        webupdate.run(['--file', androidFilePath])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 173, 'Error number should be 173');
          assert.deepEqual(err.code, 'INVALIDFILENAMEWEBUPDATE', 'Error code should be INVALIDFILENAMEWEBUPDATE');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate with non zip file --file', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var androidFilePath = path.resolve(mobilefirstPath, 'io.cordova.hellocordova-android-0.0.1.web');
        //Create a resource file under the mobilefirst dir
        fs.mkdirSync(mobilefirstPath);
        fs.writeFileSync(androidFilePath, 'androidFile');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(androidFilePath), true, 'android file we just created should exist');
        webupdate.run(['--file', androidFilePath])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 170, 'Error number should be 170');
          assert.deepEqual(err.code, 'UPDATEFILENOTVALIDZIPFILE', 'Error code should be UPDATEFILENOTVALIDZIPFILE');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate with nonexistent file using --file', function (done) {
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');
        var androidFilePath = path.resolve(mobilefirstPath, 'io.cordova.hellocordova-android-0.0.1.zip');

        process.chdir(projPath);

        assert.deepEqual(validateFileOrFolderExists(androidFilePath), false, 'android file should not exist');
        webupdate.run(['--file', androidFilePath])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 141, 'Error number should be 141');
          assert.deepEqual(err.code, 'FILENOEXISTS', 'Error code should be FILENOEXISTS');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate when --file is used with --build', function (done) {
        var androidPath = path.resolve(projPath, 'platforms', 'android');
        var mobilefirstPath = path.resolve(projPath, 'mobilefirst');

        process.chdir(androidPath);

        assert.deepEqual(validateFileOrFolderExists(mobilefirstPath), false, 'mobilefirst dir should not exist');
        webupdate.run(['--file', 'myfile.zip', '--build'])
        .then(function () {
          assert(false, 'should not have been successful with the --file and --build options');
        },function (err) {
          assert.notDeepEqual(err, undefined, 'Error should not be undefined');
          assert.deepEqual(err.errno, 171, 'Should have reported back an error no. 171');
          assert.deepEqual(err.code, 'FILESPECIFIEDWITHBUILD', 'Should have got back error FILESPECIFIEDWITHBUILD');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('should not perform webupdate when in native project', function (done) {
        nativeFixtureProject.clean();
        var nativeProjPath = path.resolve(fixturePath, 'IOSNativeProject');

        assert.deepEqual(validateFileOrFolderExists(nativeProjPath), true, 'native project should exist');
        var originalCwd = process.cwd();
        process.chdir(nativeProjPath);

        webupdate.run([])
        .then(function () {
          assert(false, 'should not have been successful in native project');
        },function (err) {
          assert.notDeepEqual(err, undefined, 'Error should not be undefined');
          assert.deepEqual(err.errno, 188, 'Should have reported back an error no. 171');
          assert.deepEqual(err.code, 'MUSTBECORDOVAPROJECT', 'Should have got back error MUSTBECORDOVAPROJECT');
          process.chdir(originalCwd);
          nativeFixtureProject.delete();
          done();
        })
        .catch(function (err) {
          process.chdir(originalCwd);
          nativeFixtureProject.delete();
          done(err);
        });
    }),

    it('should not perform webupdate if no platforms are detected', function (done) {
        process.chdir(__dirname);

        webupdate.run(['--file', 'myfile.zip', ])
        .then(function () {
          assert(false, 'should not have been successful with the --file and --build options');
        },function (err) {
          assert.notDeepEqual(err, undefined, 'Error should not be undefined');
          assert.deepEqual(err.errno, 177, 'Should have reported back an error no. 177');
          assert.deepEqual(err.code, 'NOTINAVALIDPROJECT', 'Should have got back error NOTINAVALIDPROJECT');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    });

    function preparePass (platform) {
      preparePassed = true;
      preparePlatforms.push(platform);
      return createReturnPromise();
    }

    function checksumPass (configXMLDir, platform) {
      checksumPassed = true;
      checksumPlatforms.push(platform);
      return createReturnPromise();
    }

    function createReturnPromise() {
      var d = Q.defer();
      d.resolve();

      return d.promise;
    }
});

function verifyContentOfResourceFile(filepath, platform) {
  var zip = new admZip(filepath);
  var zipEntries = zip.getEntries();
  var validZip = true;

  if(platform === 'android')
    assert.deepEqual(zipEntries.length, 21, 'number of files copied for Android zip');
  else if(platform === 'ios')
    assert.deepEqual(zipEntries.length, 15, 'number of files copied for IOS zip');
  else
    assert(false, 'we got a platform other than android or ios');

  var filesArray = ['meta/', 'meta/deployment.data', 'www/', 'www/cordova-js-src/', 'www/cordova-js-src/android/', 'www/cordova-js-src/android/nativeapiprovider.js', 'www/cordova-js-src/android/promptbasednativeapi.js', 'www/cordova-js-src/exec.js', 'www/cordova-js-src/platform.js', 'www/cordova-js-src/plugin/', 'www/cordova-js-src/plugin/android/', 'www/cordova-js-src/plugin/android/app.js'];
  filesArray = filesArray.concat(['www/cordova.js', 'www/cordova_plugins.js', 'www/css/', 'www/css/index.css', 'www/img/', 'www/img/logo.png', 'www/index.html', 'www/js/', 'www/js/index.js']);
  zipEntries.forEach(function (entry) {
    if(filesArray.indexOf(entry.entryName) < 0) {
      validZip = false;
    }
  });

  return validZip;
}

function removeMobileFirstDir(projectPath) {
  var mobilefirstDir = path.resolve(projectPath, 'mobilefirst');
  rimraf.sync(mobilefirstDir);
}
