/*
	Licensed Material - Property of IBM

	(C) Copyright 2015 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var assert = require('assert');
var path = require('path');
var findExtensions = require('pluggington');
var _getAllExtensions = findExtensions._getAllExtensions;
//this is hokey but it has to be here.
var pkg = require('../../mfpdev-cli/package.json');
global.MDO_CLI_FULL_VERSION = pkg.productVersion+ '-' +pkg.version.split('.')[2];

var config = require(path.join('..', 'lib', 'global.js'));
var home = require('user-home');
var q = require('q');
var pathToConfig = path.join(String(home), ".ibm", "mobilefirst", global.MDO_CLI_FULL_VERSION, "config_cli.json");
var fs = require('fs');
var config_api = require('../lib/config_api.js');

describe('mdo-cmd-config', function() {
  var rename = q.nfbind(fs.rename);

  // if an existing config file is found, store the path here
  var configToRestore;

  /*
   * If a config file exists it backs it up and creates a test config;
   * If it does not exist then it will make one and a tester and delete the tester
   */
  before(function(done){
    if(fs.existsSync(pathToConfig)) {
      configToRestore = pathToConfig + '.old';

      rename(pathToConfig, configToRestore)
      .then(function(){
        // create a brand new config file
        return config_api.verifyConfig();
      })
      .then(function(){
        done();
      }, function(err){
        done(err);
      });
    } else {
      config_api.verifyConfig().then(function(){
        done();
      }, function(err){
        done(err);
      });
    }
  });

  it('should contribute a mdo.cli.commands extension', function() {
    var extensions = findExtensions(path.resolve(__dirname, '..', '..'), 'mdo.cli.commands');

    assert.notEqual(extensions, undefined, 'There should be a mdo.cli.commands extension here');

    var foundCommand = false;
    extensions.forEach(function(commandExtension) {
      if (commandExtension.name === "config") {
        foundCommand = true;

        assert.notEqual(commandExtension.export.run, undefined, "config command extension does not export a run function!");
        assert.notEqual(commandExtension.export.help, undefined, "config command extension does not export a help function!");
      }
    });

    assert.equal(foundCommand, true, "I didn't find the config command extension!");
  });

  it('should contribute a mdo.cli.help extension', function() {
    var extensions = _getAllExtensions(path.resolve(__dirname, '..', '..'));
    var helpExtensions = extensions['mdo.cli.help'];

    var foundHelpExtension = false;

    helpExtensions.forEach(function(extension) {
      if (extension._path.indexOf('mdo-cmd-config/package.json') > -1) {
        foundHelpExtension = true;
      }
    });

    assert(foundHelpExtension, "didn't find the config command's help extension");
  });

  it('checks to make sure config_cli.json exists in the right folder', function() {
    try {
      var file = require(pathToConfig);
      if(file) assert(true, "Found config.");
    } catch (err) {
      assert(false, "Could not find config, Error Message: "+err);
    }
  });

  it('checks to make sure config_cli.json has the right keys', function() {
    try {
      var file = require(pathToConfig);
      assert(file['development_browser'], "Didn't find 'development_browser' in config_cli.json");
      assert(file['default_preview_type'], "Didn't find 'default_preview_type' in config_cli.json");
    } catch (err) {
      assert(false, err);
    }
  });

  it('validates that it can update a key\'s value', function(done) {
    var key = 'default_preview_type';
    var oldVal = require(pathToConfig)[key].value;
    var newVal;
    if(oldVal === 'browser') newVal = 'mbs';
    else if(oldVal === 'mbs') newVal = 'browser';

    if(newVal){
      config.withKeyAndValue(key, newVal).then(
        function(){
          var checkVal = JSON.parse(fs.readFileSync(pathToConfig))[key].value;
          if(checkVal !== newVal || checkVal === oldVal) assert(false, "The value was not updated correctly.");
          done();
        },
        function(err){
          assert(false, "There was an error validating if values change. Error: "+err);
          done(err);
      });
    }else {
      assert(false, key+ ' did not have an accurate value.');
      done();
    }
  });

  it('rejects an invalid key', function(done) {
    var val = 'development_browser';
    var key = 'invalid test key';

    config.withKeyAndValue(key, val).then(function() {
      assert(false, "No error was thrown for an incorrect key");
      done();
    },function(err) {
      assert.equal(err.errno, 137, "incorrect error. Should be INVALIDCONFIGKEY.");
      done();
    });
  });

  it('rejects an invalid value', function(done) {
    var key = 'development_browser';
    var val = 'invalid test value';

    config.withKeyAndValue(key, val).then(function(){
      assert(false, "No error was thrown for an incorrect value");
      done();
    },function(err) {
      assert.equal(err.errno, 138, "incorrect error. Should be INVALIDCONFIGVALUE.");
      done();
    });
  });

  it('rejects an invalid key and value', function(done) {
    var key = 'invalid test key';
    var val = 'invalid test value';

    config.withKeyAndValue(key, val).then(function(){
      assert(false, "No error was thrown for an incorrect key or val.");
      done();
    }, function(err) {
      assert.equal(err.errno, 137, "incorrect error. Should be INVALIDCONFIGKEY.");
      done();
    });
  });

  it('checks to make sure config_api can get values', function(done) {
    config_api.getDevelopmentBrowser()
      .then(function(devBrowser) {
        assert.notEqual(devBrowser, undefined, 'Could not get Development Browser value.');
        return config_api.getDefaultPreviewType();
      })
      .then(function(defaultPreviewType){
        assert.notEqual(defaultPreviewType, undefined, 'Could not get Default Preview Type value.');
        done();
    })
      .catch(function(err){
        assert(false, 'failed to get value using api: '+err);
        done(err);
      });
  });

  after(function(done){
    var del = q.nfbind(fs.unlink);

    del(pathToConfig)
      .then(function(){
        if(configToRestore) {
          return rename(configToRestore, pathToConfig);
        } else {
          done();
        }
      })
      .then(function(){
        done();
      })
      .catch(function(err){
        assert(false, 'failed to delete tester and restore user config: '+err);
        done(err);
      });
  });
});
