'use strict';

var assert = require('assert');
var path = require('path');
var findExtensions = require('pluggington');
var mockery = require('mockery');
var Q = require('q');

describe('mdo-cmd-mfservices', function() {
  var mdoApp;
  this.timeout(50000);

  before(function() {
    global.MDO_MAIN_EXTENSIONS_PATH =
      path.join(__dirname, '../../mfpdev-cli/node_modules');

    mockery.enable({
      useCleanCache: true,
      warnOnUnregistered: false,
      warnOnReplace: true
    });      

    var serverDetailsMock = {
      getServerDetails: function () {
        var d = Q.defer();
        d.resolve({
            runtime: 'mfp',
            serverProfile: {
                host: 'templocalhost',
                protocol: 'http',
                port: '9080'
            },
            registerApplication: function (jsonPayload) {
                var d = Q.defer();
                d.resolve({
                    appInfo: jsonPayload.applicationKey.clientPlatform + ' was built',
                });

                return d.promise;
            }
        });
        return d.promise;
      },      
      profileIO: {
        getServerProfiles: function (cc, dd) {
          var d = Q.defer();
          d.resolve({
            servers: {
              "icpa-mobile-server" : {
                'protocol': "http",
                'host': "templocalhost",
                'port': "9080",
                "ipAddress": "1.2.3.4"         
              }
            }
          });
          return d.promise;
        },
        handleLocalHost: function () {
          var d = Q.defer()
          d.resolve();
          return d.promise;
        }
      }
    };

    class dockerModem {
      constructor() {        
      }
      followProgress(stream, callback) {
        callback();
      }
    }

    class dockerMock  {
      constructor () {
        this.modem = new dockerModem();
      };
      dockerMock() {
      }
      listContainers(opts, callback) {
        callback(null, [{
          Image: 'ibmcom/ibm-mobilefoundation-ce:latest',
          Id: 'test',
          State: 'running'     
        }]);
      };
      getContainer(id) {
        return {
          stop: function() {
            var d = Q.defer()
            d.resolve();
            return d.promise;
          },
          remove: function() {
            var d = Q.defer()
            d.resolve();
            return d.promise;
          }
        };
      };
      pull(image, callback)  {
        callback();        
      };    
    };    
   

    mockery.registerMock('mdo-server-support', serverDetailsMock);
    mockery.registerMock('dockerode', dockerMock);
    mdoApp = require('../lib/index.js');
  });

  it('should contribute a mdo.cli.commands extension', function() {
    var extensions = findExtensions(path.resolve(__dirname, '..', '..'), 'mdo.cli.commands');

    assert.notEqual(extensions, undefined, 'There should be a mdo.cli.commands extension here');

    var foundCommand = false;
    extensions.forEach(function(commandExtension) {
      if (commandExtension.name === "mfservices") {
        foundCommand = true;
      }
    });

    assert.equal(foundCommand, true, "I didn't find the app command extension!");
  });

  it('should throw error as the command operation is invalid', function(done) {
    mdoApp.run(['test']).then(function () {
      assert(false, 'error should have been thrown');
      done();
    }).fail(function (err) {
      assert.strictEqual(err.errno, 218, 'Expected return code 218');
      done();
    }).catch(function (err) {
      done(err);
    });
  }); 
  
  it('test start command in non client app', function(done) {
    mdoApp.run(['start']).then(function () {
      assert(false, 'error should have been thrown');
      done();
    }).fail(function (err) {
      assert.strictEqual(err.errno, 145, 'Expected return code 145');
      done();
    }).catch(function (err) {
      done(err);
    });
  });  

  describe('mdo-cmd-mfservices-ops', function() {
    before(function() {
      process.chdir(path.join('.', 'test'));
    });    

    it('test start command in client app', function(done) {
      mdoApp.run(['start']).then(function () {
        done();
      }).fail(function (err) {
        done(err);
      }).catch(function (err) {
        done(err);
      });
    });
    
    it('test stop command in client app', function(done) {
      mdoApp.run(['stop']).then(function () {
        done();
      }).fail(function (err) {
        done(err);
      }).catch(function (err) {
        done(err);
      });
    });     
  });
});
