'use strict';

var assert = require('assert');
var path = require('path');
var fs = require('fs');

// Private modules
var server = require('../../lib/index.js');
global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '../../../mfpdev-cli/node_modules');
global.MDO_CLI_FULL_VERSION = require('../../../mfpdev-cli/package.json').version + '_mdo-cmd-server';

var HOME = process.env[(process.platform === 'win32') ? 'USERPROFILE' : 'HOME'];
var IBM_HOME = path.join(HOME, '.ibm');
var MFP_HOME = path.join(IBM_HOME, 'mobilefirst');
var versionPath = path.join(MFP_HOME, global.MDO_CLI_FULL_VERSION);
var profilePath = path.join(versionPath, 'server_profiles.json');


describe('mdo server add command', function () {
  this.timeout(5000);

  before(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }
    done();
  });

  it('Run server add remoteserver test', function (done) {
    server.run(['add', 'remoteserver', '-u', 'http://remoteserver.ibm.com:9080', '-l', 'demo', '-p', 'demo', '-t', '1'])
      .then(function () {
        try {
          var serverProfiles = JSON.parse(fs.readFileSync(profilePath, 'utf8'));
          assert.notEqual(serverProfiles.servers.remoteserver, undefined);
          assert.equal(serverProfiles.servers.remoteserver.protocol, 'http');
          assert.equal(serverProfiles.servers.remoteserver.host, 'remoteserver.ibm.com');
          assert.equal(serverProfiles.servers.remoteserver.port, '9080');
          assert.equal(serverProfiles.servers.remoteserver.login, 'demo');
          assert.equal(serverProfiles.servers.remoteserver.password, 'ZGVtbw==');
          assert.equal(serverProfiles.servers.remoteserver.contextRoot, 'mfpadmin');
          assert.equal(serverProfiles.servers.remoteserver.timeout, '1');
          assert.equal(serverProfiles.defaultServer, 'remoteserver');
          done();
        }
        catch (err) {
          assert(false, 'cannot read server_profile.json');
          done(err);
        }
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run server already exists error test', function (done) {
      server.run(['add', 'remoteserver', '-u', 'http://remoteserver.ibm.com:9080', '-l', 'demo', '-p', 'demo'])
        .then(function () {
          assert(false, 'error should have been thrown');
        })
        .fail(function (rsp) {
          assert(
            rsp.errno === 119,
            'Error code: ' + rsp
          );
          done();
        })
        .catch(function (err) {
          done(err);
        });
  });

  it('Run invalid arguments server error test', function (done) {
    server.run(['add', 'remoteserver2', '-u', 'http://remoteserver.ibm.com:9080', 'demo'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 130,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run invalid server name error test', function (done) {
    server.run(['add', 'remote server3', '-u', 'http://remoteserver.ibm.com:9080', '-l', 'demo', '-p', 'demo'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 120,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run invalid server url error test', function (done) {
    server.run(['add', 'remoteserver4', '-u', 'http://remoteserver.ibm.com', '-l', 'demo', '-p', 'demo'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 129,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run cannot add server named local error test', function (done) {
    server.run(['add', 'local', '-u', 'http://localhost:9080', '-l', 'demo', '-p', 'demo'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 118,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run url is missing error test', function (done) {
    server.run(['add', 'remoteserver5', '-l', 'demo', '-p', 'demo'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 128,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
    });
  });

  it('Run timeout is invalid error test', function (done) {
    server.run(['add', 'remoteserver6', '-u', 'http://remoteserver.ibm.com:9080', '-l', 'demo', '-p', 'demo', '-t', 'one'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 117,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
    });
  });

  after(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }
    done();
  });
});
