'use strict';

var assert = require('assert');
var path = require('path');
var fs = require('fs');

var profileIO =  require('mdo-server-support').profileIO;

// Private modules
var server = require('../../lib/index.js');
global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '../../../mfpdev-cli/node_modules');
global.MDO_CLI_FULL_VERSION = require('../../../mfpdev-cli/package.json').version + '_mdo-cmd-server';

var HOME = process.env[(process.platform === 'win32') ? 'USERPROFILE' : 'HOME'];
var IBM_HOME = path.join(HOME, '.ibm');
var MFP_HOME = path.join(IBM_HOME, 'mobilefirst');
var versionPath = path.join(MFP_HOME, global.MDO_CLI_FULL_VERSION);
var profilePath = path.join(versionPath, 'server_profiles.json');


describe('mdo server edit command', function () {
  this.timeout(5000);

  before(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }

    server.run(['add', 'remoteserver', '-u', 'http://remoteserver:9080', '-l', 'demo', '-p', 'demo', '-t', '1'])
      .then(function () {
        return server.run(['add', 'remoteserver2', '-u', 'http://remoteserver2.ibm.com:9080', '-l', 'demo', '-p', 'demo', '-t', '1']);
      })
      .then(function() {
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run profile does not exist error test', function (done) {
    server.run(['edit', 'remoteserver3', '-n' , 'newname'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 115,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run cannot rename to local', function (done) {
    server.run(['edit', 'remoteserver', '-n', 'local'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 118,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run cannot rename to name that already exists', function (done) {
    server.run(['edit', 'remoteserver', '-n', 'remoteserver2'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 119,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run invalid url error', function (done) {
    server.run(['edit', 'remoteserver', '-u', 'www.ibm.com'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 129,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run invalid timeout error', function (done) {
    server.run(['edit', 'remoteserver', '-t', 'five'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 117,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run missing arguments server error test', function (done) {
    server.run(['edit', 'remoteserver2'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 130,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run invalid arguments server error test', function (done) {
    server.run(['edit', 'remoteserver2', '--badargs'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 130,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run server edit remoteserver test', function (done) {
    server.run(['edit', 'remoteserver', '-u', 'http://editedserver.ibm.com:10080', '-c', 'contextroot', '-t', '2'])
      .then(function () {
        try {
          var serverProfiles = JSON.parse(fs.readFileSync(profilePath, 'utf8'));
          assert.notEqual(serverProfiles.servers.remoteserver, undefined);
          assert.equal(serverProfiles.servers.remoteserver.protocol, 'http');
          assert.equal(serverProfiles.servers.remoteserver.host, 'editedserver.ibm.com');
          assert.equal(serverProfiles.servers.remoteserver.port, '10080');
          assert.equal(serverProfiles.servers.remoteserver.login, 'demo');
          assert.equal(serverProfiles.servers.remoteserver.password, 'ZGVtbw==');
          assert.equal(serverProfiles.servers.remoteserver.contextRoot, 'contextroot');
          assert.equal(serverProfiles.servers.remoteserver.timeout, '2');
          assert.equal(serverProfiles.defaultServer, 'remoteserver');
          done();
        }
        catch (err) {
          assert(false, 'cannot read server_profile.json');
          done(err);
        }
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run server edit remoteserver2 test', function (done) {
    server.run(['edit', 'remoteserver2', '-u', 'http://editedserver2.ibm.com:10080', '-l', 'admin', '-p', 'admin', '-s'])
      .then(function () {
        return profileIO.getProfileDetails('remoteserver2', true);
      })
      .then(function (profile) {
        assert.notEqual(profile, undefined);
        assert.equal(profile.protocol, 'http');
        assert.equal(profile.host, 'editedserver2.ibm.com');
        assert.equal(profile.port, '10080');
        assert.equal(profile.login, 'admin');
        assert.equal(profile.password, 'YWRtaW4=');
        assert.equal(profile.contextRoot, 'mfpadmin');
        assert.equal(profile.timeout, '1');
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  after(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }
    done();
  });
});
