'use strict';

var assert = require('assert');
var path = require('path');
var fs = require('fs');

// Private modules
var server = require('../../lib/index.js');
global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '../../../mfpdev-cli/node_modules');
global.MDO_CLI_FULL_VERSION = require('../../../mfpdev-cli/package.json').version + '_mdo-cmd-server';

var HOME = process.env[(process.platform === 'win32') ? 'USERPROFILE' : 'HOME'];
var IBM_HOME = path.join(HOME, '.ibm');
var MFP_HOME = path.join(IBM_HOME, 'mobilefirst');
var versionPath = path.join(MFP_HOME, global.MDO_CLI_FULL_VERSION);
var profilePath = path.join(versionPath, 'server_profiles.json');


describe('mdo server remove command', function () {
  this.timeout(5000);

  before(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }

    server.run(['add', 'remoteserver', '-u', 'http://remoteserver:9080', '-l', 'demo', '-p', 'demo', '-t', '1'])
      .then(function () {
        return server.run(['add', 'remoteserver2', '-u', 'http://remoteserver2.ibm.com:9080', '-l', 'demo', '-p', 'demo', '-t', '1']);
      })
      .then(function() {
        done();
      })
      .catch(function () {
        done();
    });
  });

  it('Run server remove remoteserver2 test', function (done) {
   try {
     var serverProfiles = JSON.parse(fs.readFileSync(profilePath, 'utf8'));
     assert.notEqual(serverProfiles.servers.remoteserver, undefined);
     assert.notEqual(serverProfiles.servers.remoteserver2, undefined);
     assert.equal(serverProfiles.defaultServer, 'remoteserver');
     server.run(['remove', 'remoteserver2'])
       .then(function () {
         serverProfiles = JSON.parse(fs.readFileSync(profilePath, 'utf8'));
         assert.notEqual(serverProfiles.servers.remoteserver, undefined);
         assert.equal(serverProfiles.servers.remoteserver2, undefined);
         assert.equal(serverProfiles.defaultServer, 'remoteserver');
         done();
         }
       )
       .catch(function (err) {
         done(err);
       });
   }
   catch (err) {
     assert(false, 'cannot read server_profile.json');
     done(err);
   }
  });

  it('Run server remove remoteserver error test', function (done) {
    server.run(['remove', 'remoteserver'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 123,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run server remove remoteserver3 not found error test', function (done) {
    server.run(['remove', 'remoteserver3'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 115,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run server remove remoteserver invalid args error test', function (done) {
    server.run(['remove', 'remoteserver2', 'demo'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 130,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run server remove no profiles to remove error test', function (done) {
    server.run(['remove'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 116,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  it('Run invalid server name error test', function (done) {
    server.run(['remove', 'remote server3'])
      .then(function () {
        assert(false, 'error should have been thrown');
      })
      .fail(function (rsp) {
        assert(
          rsp.errno === 120,
          'Error code: ' + rsp
        );
        done();
      })
      .catch(function (err) {
        done(err);
      });
  });

  after(function(done){
    if (fs.existsSync(profilePath)) {
      try {
        fs.unlinkSync(profilePath);
      }
      catch (err) {
        assert(false, 'cannot delete server_profile.json');
      }
    }
    done();
  });

});
