/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach*/
var assert = require('assert');
var fs = require('fs');
var log = require('npmlog');
var mockery = require('mockery');
var path = require('path');
var Q = require('q');
//private
var getTestFixture = require('mdo-test-resources');

describe('mdo-cordova-support checksum', function () {
  this.timeout(10000);
    var fixturePath,
        fixtureProject,
        checksum,
        execRan = false;

    global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

    before(function () {
      fixturePath = path.resolve(__dirname, '..', 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);

      fixtureProject.unzip();
      //Set up the intial functions we are mocking
      mockery.enable({
          warnOnUnregistered: false,
          warnOnReplace: true
      });

      var mdoExecMock = {
          run: function (command, args) {
            var d = Q.defer();
            execRan = true;
            d.resolve({
              command: command,
              args: args
            });
            return d.promise;
          }
      };

      //Set up a way to control the log.level from the tests, it has to happen before we require our chekcsum module
      mockery.registerMock('mdo-exec', mdoExecMock);
    });

    beforeEach(function () {
      execRan = false;
    });

    after(function () {
      fixtureProject.delete();
      mockery.disable();
    });

    it('the checksum code should thrown an error if the plugin is not found', function (done) {
        log.level = 'info';
        checksum = loadChecksumModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');

        assert.deepEqual(validateFileOrFolderExists(pluginPath), false, 'the mfp plugin folder should not exist');

        checksum(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 174, 'error number should be 174');
          assert.deepEqual(err.code, 'CORDOVAMFPPLUGINNOTINSTALLED', 'error should be that mfp plugin was not installed');
          assert.deepEqual(execRan, false, 'exec command should not have ran');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the checksum code should thrown an error if the checksum script is not found', function (done) {
        log.level = 'info';
        checksum = loadChecksumModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');

        assert.deepEqual(validateFileOrFolderExists(pluginPath), false, 'the mfp plugin folder should not exist');
        fs.mkdirSync(pluginPath);
        assert.deepEqual(validateFileOrFolderExists(pluginPath), true, 'the mfp plugin folder should not exist');

        checksum(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 175, 'error number should be 175');
          assert.deepEqual(err.code, 'SCRIPTNOTPRESENT', 'error should be that the checksum script was not found');
          assert.deepEqual(execRan, false, 'exec command should not have ran');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the checksum code should exec the checksum script if proper command and args, no debug args', function (done) {
        log.level = 'info';
        checksum = loadChecksumModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');
        var checksumScriptPath = path.resolve(pluginPath, 'lib', 'mfp-checksum-cli.js');

        assert.deepEqual(validateFileOrFolderExists(pluginPath), true, 'the mfp plugin folder should exist');
        assert.deepEqual(validateFileOrFolderExists(checksumScriptPath), false, 'the checksum script should not exist');
        fs.mkdirSync(path.resolve(pluginPath, 'lib'));
        fs.writeFileSync(checksumScriptPath, 'checksum');
        assert.deepEqual(validateFileOrFolderExists(checksumScriptPath), true, 'the checksum script should now exist');

        checksum(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function (result) {
          assert.deepEqual(execRan, true, 'checksum script should have been execed');
          assert.deepEqual(result.command, 'node', 'execed command should have bee node');
          var execArgs = [checksumScriptPath, '--path', projPath, '--platforms', 'android'];
          assert.deepEqual(result.args, execArgs, 'args should have been properly sent');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the checksum code should exec the checksum script if proper command and args, verbose debug args', function (done) {
        log.level = 'verbose';
        checksum = loadChecksumModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');
        var checksumScriptPath = path.resolve(pluginPath, 'lib', 'mfp-checksum-cli.js');

        checksum(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function (result) {
          assert.deepEqual(execRan, true, 'checksum script should have been execed');
          assert.deepEqual(result.command, 'node', 'execed command should have bee node');
          var execArgs = [checksumScriptPath, '--path', projPath, '--platforms', 'android', '-d'];
          assert.deepEqual(result.args, execArgs, 'args should have been properly sent');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the checksum code should exec the checksum script if proper command and args, silly debug args', function (done) {
        log.level = 'silly';
        checksum = loadChecksumModule();
        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');
        var checksumScriptPath = path.resolve(pluginPath, 'lib', 'mfp-checksum-cli.js');

        checksum(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function (result) {
          assert.deepEqual(execRan, true, 'checksum script should have been execed');
          assert.deepEqual(result.command, 'node', 'execed command should have bee node');
          var execArgs = [checksumScriptPath, '--path', projPath, '--platforms', 'android', '-dd'];
          assert.deepEqual(result.args, execArgs, 'args should have been properly sent');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    });
});

function validateFileOrFolderExists(path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
}

function loadChecksumModule() {
  var name = require.resolve('../../lib/generateChecksum');
  delete require.cache[name];

  return require('../../lib/generateChecksum');
}
