/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach*/
var assert = require('assert');
var fs = require('fs');
var log = require('npmlog');
var mockery = require('mockery');
var path = require('path');
var Q = require('q');
//private
var getTestFixture = require('mdo-test-resources');

describe('mdo-cordova-support encrypt', function () {
  this.timeout(10000);
    var fixturePath,
        fixtureProject,
        encrypt,
        execRan = false;

    global.MDO_MAIN_EXTENSIONS_PATH = path.join(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

    before(function () {
      fixturePath = path.resolve(__dirname, '..', 'fixtures');
      fixtureProject = getTestFixture('CordovaProject.zip', fixturePath);

      fixtureProject.unzip();
      //Set up the intial functions we are mocking
      mockery.enable({
          warnOnUnregistered: false,
          warnOnReplace: true
      });

      var mdoExecMock = {
          run: function (command, args) {
            var d = Q.defer();
            execRan = true;
            d.resolve({
              command: command,
              args: args
            });
            return d.promise;
          }
      };

      mockery.registerMock('mdo-exec', mdoExecMock);
    });

    beforeEach(function () {
      execRan = false;
    });

    after(function () {
      fixtureProject.delete();
      mockery.disable();
    });


    it('the encrypt code should thrown an error if the plugin is not found', function (done) {
        log.level = 'info';
        encrypt = loadEncryptModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');

        assert.deepEqual(validateFileOrFolderExists(pluginPath), false, 'the mfp plugin folder should not exist');

        encrypt(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 174, 'error number should be 174');
          assert.deepEqual(err.code, 'CORDOVAMFPPLUGINNOTINSTALLED', 'error should be that mfp plugin was not installed');
          assert.deepEqual(execRan, false, 'exec command should not have ran');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the encrypt code should thrown an error if the encrypt script is not found', function (done) {
        log.level = 'info';
        encrypt = loadEncryptModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');

        assert.deepEqual(validateFileOrFolderExists(pluginPath), false, 'the mfp plugin folder should not exist');
        fs.mkdirSync(pluginPath);
        assert.deepEqual(validateFileOrFolderExists(pluginPath), true, 'the mfp plugin folder should not exist');

        encrypt(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function () {
          assert(false, 'should not have entered success function');
        }, function (err) {
          assert.notDeepEqual(err, undefined, 'err should not be undefined');
          assert.deepEqual(err.errno, 175, 'error number should be 175');
          assert.deepEqual(err.code, 'SCRIPTNOTPRESENT', 'error should be that the encrypt script was not found');
          assert.deepEqual(execRan, false, 'exec command should not have ran');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the encrypt code should exec the encrypt script if proper command and args, no debug args', function (done) {
        log.level = 'info';
        encrypt = loadEncryptModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');
        var encryptScriptPath = path.resolve(pluginPath, 'lib', 'mfp-enc-cli.js');

        assert.deepEqual(validateFileOrFolderExists(pluginPath), true, 'the mfp plugin folder should exist');
        assert.deepEqual(validateFileOrFolderExists(encryptScriptPath), false, 'the encrypt script should not exist');
        fs.mkdirSync(path.resolve(pluginPath, 'lib'));
        fs.writeFileSync(encryptScriptPath, 'encrypt');
        assert.deepEqual(validateFileOrFolderExists(encryptScriptPath), true, 'the encrypt script should now exist');

        encrypt(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function (result) {
          assert.deepEqual(execRan, true, 'encrypt script should have been execed');
          assert.deepEqual(result.command, 'node', 'execed command should have bee node');
          var execArgs = [encryptScriptPath, '--path', projPath, '--platforms', 'android'];
          assert.deepEqual(result.args, execArgs, 'args should have been properly sent');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the encrypt code should exec the encrypt script if proper command and args, silly debug args', function (done) {
        log.level = 'verbose';
        encrypt = loadEncryptModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');
        var encryptScriptPath = path.resolve(pluginPath, 'lib', 'mfp-enc-cli.js');


        encrypt(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function (result) {
          assert.deepEqual(execRan, true, 'encrypt script should have been execed');
          assert.deepEqual(result.command, 'node', 'execed command should have bee node');
          var execArgs = [encryptScriptPath, '--path', projPath, '--platforms', 'android', '-d'];
          assert.deepEqual(result.args, execArgs, 'args should have been properly sent');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    }),

    it('the encrypt code should exec the encrypt script if proper command and args, silly debug args', function (done) {
        log.level = 'silly';
        encrypt = loadEncryptModule();

        var projPath = path.resolve(fixturePath, 'CordovaProject');
        var pluginPath = path.resolve(projPath, 'plugins', 'cordova-plugin-mfp');
        var encryptScriptPath = path.resolve(pluginPath, 'lib', 'mfp-enc-cli.js');


        encrypt(projPath, ['--path', projPath, '--platforms', 'android'])
        .then(function (result) {
          assert.deepEqual(execRan, true, 'encrypt script should have been execed');
          assert.deepEqual(result.command, 'node', 'execed command should have bee node');
          var execArgs = [encryptScriptPath, '--path', projPath, '--platforms', 'android', '-dd'];
          assert.deepEqual(result.args, execArgs, 'args should have been properly sent');
          done();
        })
        .catch(function (err) {
          done(err);
        });
    });
});

function validateFileOrFolderExists(path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
}


function loadEncryptModule() {
  var name = require.resolve('../../lib/encryptWWWFolder');
  delete require.cache[name];

  return require('../../lib/encryptWWWFolder');
}
