/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var fs = require('fs');
var path = require('path');
//private packages
var cordovaPlatform = require('../../');
var ConfigXMLAPI = require('mfp-config-xml').mfpConfigXMLAPI;

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

var verifyConfigXMLOriginalState = function (configXMLAPI) {
    assert.deepEqual(configXMLAPI.getMFPServerURL(), 'http://10.0.0.1:9080');
    assert.deepEqual(configXMLAPI.getMFPServerRuntime(), 'mfp');
    assert.deepEqual(configXMLAPI.getMFPPlatformVersion(), '');
    assert.deepEqual(configXMLAPI.getMFPDirectUpdateAuthKey(), '');
    assert.deepEqual(configXMLAPI.getMFPLanguagePrefs(), 'en');
    assert.deepEqual(configXMLAPI.getMFPClientCustomInit(), 'false');
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('android'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('ios'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('windows'), 0);
    var platforms = ['android', 'ios', 'windows8', 'windowsphone8', 'windows10'];
    //Check all other properties, if set do nothing, else set them

    for(var i = 0; i < platforms.length; i++) {
        assert.deepEqual(configXMLAPI.getMFPSDKChecksum(platforms[i]), 0);
        assert.deepEqual(configXMLAPI.getMFPTestWebResources(platforms[i]), 'false');
        assert.deepEqual(configXMLAPI.getMFPIgnoreFileExtensions(platforms[i]), 'png, jpg, jpeg, gif, mp4, mp3');
    }
};

var verifyConfigXML = function (configXMLAPI) {
    assert.deepEqual(configXMLAPI.getMFPServerURL(), 'http://9.12.34.56:5050');
    assert.deepEqual(configXMLAPI.getMFPServerRuntime(), 'myRuntime');
    assert.deepEqual(configXMLAPI.getMFPPlatformVersion(), '');
    assert.deepEqual(configXMLAPI.getMFPDirectUpdateAuthKey(), '');
    assert.deepEqual(configXMLAPI.getMFPLanguagePrefs(), 'en');
    assert.deepEqual(configXMLAPI.getMFPClientCustomInit(), 'false');
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('android'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('ios'), 0);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('windows'), 0);
    var platforms = ['android', 'ios', 'windows8', 'windowsphone8', 'windows10'];
    //Check all other properties, if set do nothing, else set them

    for(var i = 0; i < platforms.length; i++) {
        assert.deepEqual(configXMLAPI.getMFPSDKChecksum(platforms[i]), 0);
        assert.deepEqual(configXMLAPI.getMFPTestWebResources(platforms[i]), 'false');
        assert.deepEqual(configXMLAPI.getMFPIgnoreFileExtensions(platforms[i]), 'png, jpg, jpeg, gif, mp4, mp3');
    }
};

var verifyConfigXMLDifferentValues = function (configXMLAPI) {
    assert.deepEqual(configXMLAPI.getMFPServerURL(), 'http://9.12.34.56:5050');
    assert.deepEqual(configXMLAPI.getMFPServerRuntime(), 'myRuntime');
    assert.deepEqual(configXMLAPI.getMFPPlatformVersion(), '1.2.3');
    assert.deepEqual(configXMLAPI.getMFPDirectUpdateAuthKey(), 'myKey');
    assert.deepEqual(configXMLAPI.getMFPLanguagePrefs(), 'fr');
    assert.deepEqual(configXMLAPI.getMFPClientCustomInit(), 'true');
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('android'), 2);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('ios'), 2);
    assert.deepEqual(configXMLAPI.getMFPAppChecksum('windows'), 2);
    var platforms = ['android', 'ios', 'windows8', 'windowsphone8', 'windows10'];
    //Check all other properties, if set do nothing, else set them

    for(var i = 0; i < platforms.length; i++) {
        assert.deepEqual(configXMLAPI.getMFPSDKChecksum(platforms[i]), 55);
        assert.deepEqual(configXMLAPI.getMFPTestWebResources(platforms[i]), 'true');
        assert.deepEqual(configXMLAPI.getMFPIgnoreFileExtensions(platforms[i]), 'mkv, avi');
    }
};

var setConfigXMLToDifferentValues = function (configXMLAPI) {
    configXMLAPI.setMFPServerURL('http://1.2.3.4:1234');
    configXMLAPI.setMFPServerRuntime('different');
    configXMLAPI.setMFPPlatformVersion('1.2.3');
    configXMLAPI.setMFPDirectUpdateAuthKey('myKey');
    configXMLAPI.setMFPLanguagePrefs('fr');
    configXMLAPI.setMFPClientCustomInit('true');
    configXMLAPI.setMFPAppChecksum(2, 'android');
    configXMLAPI.setMFPAppChecksum(2, 'ios');
    configXMLAPI.setMFPAppChecksum(2, 'windows');
    var platforms = ['android', 'ios', 'windows8', 'windowsphone8', 'windows10'];
    //Check all other properties, if set do nothing, else set them

    for(var i = 0; i < platforms.length; i++) {
        configXMLAPI.setMFPSDKChecksum(55, platforms[i]);
        configXMLAPI.setMFPTestWebResources('true', platforms[i]);
        configXMLAPI.setMFPIgnoreFileExtensions('mkv, avi', platforms[i]);
    }

    configXMLAPI.writeToFile();
};

describe('mdo-cordova-support processPropertiesFile', function () {
  var cordovaProjPath,
    propertyFilePath,
    getTestFixture = require('mdo-test-resources'),
    fixturePath,
    cordovaProject;

  before(function () {
      fixturePath = path.resolve(__dirname, '..', 'fixtures');
      cordovaProject = getTestFixture('CordovaProject.zip', fixturePath);

      cordovaProject.clean();
      cordovaProjPath = path.resolve(fixturePath, 'CordovaProject');
      propertyFilePath = path.resolve(cordovaProjPath, 'config.xml');
  });

  after(function () {
      cordovaProject.delete();
  });

  it('should set mfp values in config.xml if not present', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'config.xml should have been created');
      var configXML = new ConfigXMLAPI(propertyFilePath);
      verifyConfigXMLOriginalState(configXML);
      var serverObj = 'http://9.12.34.56:5050',
          runtime = 'myRuntime';

      cordovaPlatform.processPropertiesFile(propertyFilePath, serverObj, runtime)
      .then(function () {
         configXML = new ConfigXMLAPI(propertyFilePath);
         verifyConfigXML(configXML);
         done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should only overwrite server/runtime in config.xml when file exists', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'config.xml should have been created');
      var configXML = new ConfigXMLAPI(propertyFilePath);
      setConfigXMLToDifferentValues(configXML);

      var serverObj = 'http://9.12.34.56:5050',
          runtime = 'myRuntime';

     cordovaPlatform.processPropertiesFile(propertyFilePath, serverObj, runtime)
     .then(function () {
       configXML = new ConfigXMLAPI(propertyFilePath);
       verifyConfigXMLDifferentValues(configXML);
       done();
     }).catch(function (err) {
        done(err);
     });
  });
});
