/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var fs = require('fs');
var glob = require('glob');
var path = require('path');
var plist = require('plist');
//private packages
var iosPlatform = require('../../lib/nativeIOS');
var getTestFixture = require('mdo-test-resources');
var Fixture = require('ibm-test-fixtures');

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

var fixturePath,
  iosNativeProject,
  propertyFilePath,
  iosProjPath;


describe('platform-native-ios test process property files', function () {

  before(function() {
    fixturePath = path.resolve(__dirname, '..', 'fixtures');
    iosNativeProject = getTestFixture('IOSNativeProject.zip', fixturePath);

    iosNativeProject.clean();
    iosProjPath = path.resolve(fixturePath, 'IOSNativeProject');
    propertyFilePath = path.join(iosProjPath, 'mfpclient.plist');
  });

  after(function() {
    iosNativeProject.delete();
  });

  it('should build mfpclient.plist when not present', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(iosProjPath), 'IOSNativeProject should exist');
      assert.deepEqual(false, validateFileOrFolderExists(propertyFilePath), 'mfpclient.plist should not exist yet at root');

      var platformObj = {platformPath: iosProjPath};
      var serverObj = {
          protocol: 'http',
          host: '9.12.34.56',
          port: '5050',
      };
      var runtime = 'myRuntime';

      iosPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.plist should exist now at root');
        var mfpClientFile = fs.readFileSync(propertyFilePath);
        var mfpClientData = plist.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['protocol'], 'http');
        assert.deepEqual(mfpClientData['host'], '9.12.34.56');
        assert.deepEqual(mfpClientData['port'], '5050');
        assert.deepEqual(mfpClientData['wlServerContext'], '/myRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update mfpclient.plist when file exists and server info is different', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(iosProjPath), 'IOSNativeProject should exist');
      assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.plist should exist at root');

      var platformObj = {platformPath: iosProjPath};
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: '9090',
      };
      var runtime = 'altRuntime';

      iosPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.plist should exist now at root');
        var mfpClientFile = fs.readFileSync(propertyFilePath);
        var mfpClientData = plist.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['protocol'], 'https');
        assert.deepEqual(mfpClientData['host'], '1.2.3.4.5');
        assert.deepEqual(mfpClientData['port'], '9090');
        assert.deepEqual(mfpClientData['wlServerContext'], '/altRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update port when it is a Number', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(iosProjPath), 'IOSNativeProject should exist');
      assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.plist should exist at root');

      var platformObj = {platformPath: iosProjPath};
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: 9010,
      };
      var runtime = 'altRuntime';

      iosPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.plist should exist now at root');
        var mfpClientFile = fs.readFileSync(propertyFilePath);
        var mfpClientData = plist.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['protocol'], 'https');
        assert.deepEqual(mfpClientData['host'], '1.2.3.4.5');
        assert.deepEqual(mfpClientData['port'], '9010');
        assert.deepEqual(mfpClientData['wlServerContext'], '/altRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update mfpclient.plist when file exists and file is in a different location', function (done) {
      var clientFileProject = path.resolve(__dirname, '..', 'fixtures', 'IOSNativeProjectClientFileExists.zip');
      //Move the file
      var clientFileFixture = new Fixture(clientFileProject, fixturePath);
      clientFileFixture.clean();
      var iosProjAltPath = path.resolve(fixturePath, 'IOSNativeProjectClientFileExists', 'TestNativeProject');

      var propertyFileAltPath = path.resolve(iosProjAltPath, 'Resources', 'mfpclient.plist');
      assert.deepEqual(true, validateFileOrFolderExists(iosProjAltPath), 'IOSNativeProject should exist');
      assert.deepEqual(true, validateFileOrFolderExists(propertyFileAltPath), 'mfpclient.plist should exist somewhere other than root');
      var platformObj = {platformPath: iosProjAltPath};
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: '9090',
      };
      var runtime = 'altRuntime';

      iosPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        var clientFile = glob.sync('**/mfpclient.plist', {cwd: iosProjAltPath});
        assert.deepEqual(clientFile.length, 1, 'should have found only one file');

        assert.deepEqual(true, validateFileOrFolderExists(propertyFileAltPath), 'mfpclient.plist should exist now under non-root folder');
        var mfpClientFile = fs.readFileSync(propertyFileAltPath);
        var mfpClientData = plist.parse(mfpClientFile.toString());
        assert.deepEqual(mfpClientData['protocol'], 'https');
        assert.deepEqual(mfpClientData['host'], '1.2.3.4.5');
        assert.deepEqual(mfpClientData['port'], '9090');
        assert.deepEqual(mfpClientData['wlServerContext'], '/altRuntime/');
        assert.deepEqual(mfpClientData['languagePreferences'], 'en');
        clientFileFixture.delete();
        done();
      }).catch(function (err) {
        clientFileFixture.delete();
        done(err);
      });
  });
});
