/*
	Licensed Material - Property of IBM

	(C) Copyright 2015 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


/* globals describe, it */  // make jshint happy

'use strict';
var assert = require('assert');
//var getApplications = require('../lib/applications-get');
var getApplications = require('../lib/rest-client-api/rest-client-api.js').getApplications;
var http = require('http');
var fs = require('fs');
var httpserver = null;
var debuglevel = 'info';

// Mocha apparently does not have a good way to pass in test arguments.
// The presence of a test.debugon file in the mfp-runtimeadmin-app directory
// will cause debug trace to be output to stdout.  This will not adversely
// affect the test results.
if (fs.existsSync('test.debugon')) {
    debuglevel = 'silly';
}

describe('getApplications REST test', function () {

  it('getApplications without runtime', function (done) {
    getApplications( {"debuglevel":debuglevel} )
    .then( function() {
        assert(false, 'Missing runtime argument did not result in a failure.');
        done();
     } )
    .fail( function(err) {
        assert(err.name === 'InvalidArgumentError', 'Missing runtime argument did not result in InvalidArgumentError.');
        done();
     } );
  });

  it('getApplications on stopped server - simulated via host that is not listening on specified port', function (done) {
    this.timeout(5000);
    stopServer();
    getApplications(
        {"runtime":"myproj",
         "host":"localhost",
         "port":10088,
         "debuglevel":debuglevel} )
    .then( function() {
        assert(false, 'Accessing a stopped server did not result in a failure.');
        done();
     } )
    .fail( function(err) {
        try {
            assert(err.message, 'Accessing a stopped server did not return an expected error message.');
            assert(typeof err.message === 'string', 'Accessing a stopped server did not return an expected error message as a String.');
            assert( (err.message.indexOf('ECONNREFUSED') > 0), 'Accessing a stopped server did not return an ECONNREFUSED error message.');
            assert((err.name === 'GetApplicationsError'), 'Accessing a stopped server not result in a GetApplicationsError error.');
            done();
        }
        catch(e) {
            done(e);
        }
     } );
  });

  it('getApplications on running server with unregistered runtime', function (done) {
    this.timeout(5000);
    startServer(
       { "port":10088,
         "respHttpCode":404,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify(
              {
                 "productVersion": "7.0.0.00-20150331-2005",
                 "message": "FWLSE3002E: The resource is not found.",
                 "additionalMessage": "FWLSE3041E: No MBean found for MobileFirst project 'myproj'. Possibly the MobileFirst runtime web application for MobileFirst project 'OptCliPro' is not running. If it is running, use JConsole to inspect the available MBeans. If it is not running, full error details are available in the log files of the server."
              })
       });
    getApplications(
        {"runtime":"myproj",
         "host":"localhost",
         "port":10088,
         "debuglevel":debuglevel} )
    .then( function() {
        assert(false, 'Specifying an unregistered runtime did not result in a failure.');
        done();
        stopServer();
     } )
    .fail( function(err) {
        try {
            assert(err.message, 'Specifying an unregistered runtime did not return an expected error message.');
            assert(typeof err.message === 'string', 'Specifying an unregistered runtime did not return an expected error message as a String.');
            assert((err.name === 'GetApplicationsError'), 'Specifying an unregistered runtime not result in a GetRuntimeError error.');
            assert( (err.message.indexOf('FWLSE3041E') > 0), 'Specifying an unregistered runtime did not return a FWLSE3041E error message.');
            done();
            stopServer();
        }
        catch(e) {
            done(e);
            stopServer();
        }
     } );
  });


  it('getApplications successful get', function (done) {
    this.timeout(5000);
    startServer(
       { "port":10088,
         "respHttpCode":200,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify(
             {
               "name":"TestHybridAndroid",
               "applications":[
                 {
                   "name":"HybridApp",
                   "displayName":"HybridApp",
                   "description":"HybridApp",
                   "platformVersion":"7.0.0.00.20150409-2201",
                   "link":"http://localhost:10080/worklightadmin/management-apis/1.0/runtimes/TestHybridAndroid/applications/HybridApp",
                   "projects":[
                     {
                       "name":"TestHybridAndroid"
                     }
                   ],
                   "appVersions":[
                     {
                       "application":
                       {
                         "name":"HybridApp",
                         "displayName":"HybridApp",
                         "description":"HybridApp",
                         "platformVersion":"7.0.0.00.20150409-2201"
                       },
                       "environment":"android",
                       "version":"1.0",
                       "deployTime":"2015-04-22T12:50:40.736Z",
                       "prevBuildTime":"2015-04-20T23:16:46.530Z",
                       "versionLocked":false,
                       "resourceHash":"8ff394e7bd603a3a13c6699ca955ca0b7306c998",
                       "resourceSize":805219,
                       "link":"http://localhost:10080/worklightadmin/management-apis/1.0/runtimes/TestHybridAndroid/applications/HybridApp/android/1.0",
                       "authenticityConfig":"NONE",
                       "buildTime":"2015-04-22T12:50:21.782Z"
                     },
                     {
                       "application":
                       {
                         "name":"HybridApp",
                         "displayName":"HybridApp",
                         "description":"HybridApp",
                         "platformVersion":"7.0.0.00.20150409-2201"
                       },
                       "environment":"common",
                       "version":"1.0",
                       "deployTime":"2015-04-22T12:50:40.736Z",
                       "versionLocked":false,
                       "resourceHash":"bbd455142bd11a279312f381efa6ef910bdbf6dc",
                       "resourceSize":255254,
                       "link":"http://localhost:10080/worklightadmin/management-apis/1.0/runtimes/TestHybridAndroid/applications/HybridApp/common/1.0",
                       "authenticityConfig":"NONE",
                       "buildTime":"2015-04-22T12:50:28.848Z"
                     }
                   ],
                   "pushTags":[]
                 }
               ],
               "adapters":[],
               "synchronizationStatus":"ok",
               "numberOfActiveDevices":0,
               "numberOfDecommissionedDevices":0,
               "isDeviceTabEnabled":true,
               "isAccessManagementEnabled":false,
               "analyticsConsolePath":"http://9.27.146.109:10080/analytics/console",
               "bitlyApiKey":"",
               "bitlyUsername":"",
               "platformVersion":"7.0.0.00.20150409-2201",
               "serverVersion":"7.0.0.00.20150409-2201",
               "developmentMode":true,
               "deviceTrackingEnabled":true,
               "license":
               {
                 "numberOfActiveDevices":0,
                 "numberOfInactiveDevices":0,
                 "numberOfApplications":1,
                 "numberOfServers":0,
                 "lastReportDate":"2015-04-22T12:50:57.240Z",
                 "decommissionTaskInterval":86400,
                 "decommissionWhen":90,
                 "maxNumberOfRecordedDevices":0,
                 "maxNumberOfRecordedDevicesOnDate":"2015-04-20T23:15:50.002Z",
                 "addressableDeviceUndefined":0,
                 "addressableDeviceB2E":0,
                 "addressableDeviceB2C":0,
                 "archiveDecommissionWhen":90
               },
               "running":true,
               "auditEnabled":true,
               "topology":"STANDALONE",
               "productVersion":"7.0.0.00-20150409-2201"
             })
       });
    getApplications(
        {"runtime":"TestHybridAndroid",
         "host":"localhost",
         "port":10088,
         "details":true,
         "debuglevel":debuglevel} )
    .then( function() {
        assert(true, 'Successful retrieval.');
        done();
        stopServer();
     } )
    .fail( function() {
        try {
            assert(false, 'Successful get resulted in a failure.');
            done();
            stopServer();
        }
        catch(e) {
            done(e);
            stopServer();
        }
     } );
  });

  // Starts a HTTP server used to simulate a specific HTTP response for all requests
  //
  // Parameter - single JSON object in format:
  // {
  //   port: <listen port>
  //   respHttpCode: <response HTTP status code>
  //   respHttpHeader: <JSON object.  Each attribute/value is a response header/value pair>
  //   respContent: <string or buffer containing response body>
  // }
  function startServer(options) {
      httpserver = http.createServer( function(req, res) {
          res.writeHead(options.respHttpCode, options.respHttpHeader);
          res.end(options.respContent);
      }).listen(options.port);
  }

  function stopServer() {
      if (httpserver) {
          httpserver.close();
          httpserver = null;
      }
  }

});
