/*
	Licensed Material - Property of IBM

	(C) Copyright 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


/* globals describe, it */  // make jshint happy

'use strict';
var assert = require('assert');
var getAuthToken = require('../lib/rest-client-api/rest-client-api.js').getAuthToken;

var http = require('http');
var httpserver = null;

describe('Test getAuthToken API', function () {
  this.timeout(5000);

  it('getAuthToken without runtime', function (done) {
    getAuthToken({})
    .then( function() {
        assert(false, 'Missing runtime argument did not result in a failure.');
        done();
     } )
    .fail( function(err) {
        try {
          assert(err.name === 'InvalidArgumentError', 'Missing runtime argument did not result in InvalidArgumentError.');
          done();
        }
        catch(e) {
          done(e);
        }
     });
  });

  it('getAuthToken with stopped server', function (done) {
    this.timeout(5000);
    stopServer();
    getAuthToken(
        {"host":"localhost",
         "port":10088,
         "runtime": 'mfp'} )
    .then( function() {
        assert(false, 'Accessing a stopped MFP server did not result in a failure.');
        done();
     } )
    .fail( function(err) {
        try {
            assert(err.message, 'Accessing a stopped MFP server did not return an expected error message.');
            assert(typeof err.message === 'string', 'Accessing a stopped MFP server did not return an expected error message as a String.');
            assert( (err.message.indexOf('ECONNREFUSED') > 0), 'Accessing a stopped MFP server did not return an ECONNREFUSED error message.');
            done();
        }
        catch(e) {
            done(e);
        }
     } );
  });

  it('getAuthToken with invalid token response', function (done) {
    this.timeout(5000);
    startServer(
       { "port":10088,
         "respHttpCode":404,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": ''
       });
    getAuthToken(
        {"host":"localhost",
         "port":10088,
         "runtime": "mfp"} )
    .then( function() {
        try {
          assert(false, 'Obtaining an invalid token response did not result in failure');
          done();
          stopServer();
        }
        catch(e) {
            done(e);
            stopServer();
          }
     } )
    .fail( function(err) {
        try {
            assert(err.message, 'Obtaining an invalid token response did not return an expected error message.');
            assert(err.message === 'NOTOKEN', 'Obtaining an invalid token response did not return expected NOTOKEN error');
            done();
            stopServer();
        }
        catch(e) {
            done(e);
            stopServer();
        }
     } );
  });

  it('getAuthToken with valid token response', function (done) {
    this.timeout(5000);
    startServer(
       { "port":10088,
         "respHttpCode":404,
         "respHttpHeader": { 'Content-Type':'application/json' },
         "respContent": JSON.stringify({
            "access_token": "eyJhbGciOiJSUzI1NiIsImp3ayI6eyJrdHkiOi..",
            "token_type": "Bearer",
            "expires_in": 3599,
            "scope": ""
          })
       });
    getAuthToken(
        {"host":"localhost",
         "port":10088,
         "runtime": "mfp"} )
    .then( function() {
        try {
          assert(true, 'Successful authorization token get resulted in a failure.');
          done();
          stopServer();
        }
        catch(e) {
            done(e);
            stopServer();
          }
     });
  });

  // Starts a HTTP server used to simulate a specific HTTP response for all requests
  //
  // Parameter - single JSON object in format:
  // {
  //   port: <listen port>
  //   respHttpCode: <response HTTP status code>
  //   respHttpHeader: <JSON object.  Each attribute/value is a response header/value pair>
  //   respContent: <string or buffer containing response body>
  // }
  function startServer(options) {
      httpserver = http.createServer( function(req, res) {
          res.writeHead(options.respHttpCode, options.respHttpHeader);
          res.end(options.respContent);
      }).listen(options.port);
  }

  function stopServer() {
      if (httpserver) {
          httpserver.close();
          httpserver = null;
      }
  }
});
