/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */
/*global beforeEach */

var assert = require('assert');
var fs = require('fs');
var path = require('path');
//private packages
var windowsPlatform = require('../../lib/nativeWindows10');
var getTestFixture = require('mdo-test-resources');

describe('platform-native-windows detect-native windows10', function () {
  /**Folder structure A is when the .sln file inside of the App folder
  /*  Dir structure
      Windows10NativeProjectAltStructure/
        App2/
          App2.sln
          package.appxmanifest
          ...app files...
  */
  describe('should detect windows10 with folder structure A', function () {
    var fixturePath,
        nativeProjectFixture,
        windowsProjPath;

    before(function() {
        fixturePath = path.resolve(__dirname, '..', 'fixtures');
        nativeProjectFixture = getTestFixture('Windows10NativeProject.zip', fixturePath);

        nativeProjectFixture.clean();
        windowsProjPath = path.resolve(fixturePath, 'Windows10NativeProject', 'App2');
    });

    after(function() {
      nativeProjectFixture.delete();
    });

    it('should detect native-windows 10 platform at root of project', function (done) {
        process.chdir(windowsProjPath);
        windowsPlatform.detect()
        .then(function (result) {
          assert.notEqual(result, undefined);
          assert.deepEqual(result.appName, 'App2');
          assert.deepEqual(result.platformId, 'native-windows10');
          assert.deepEqual(result.platform, 'windows');
          assert.deepEqual(result.configXMLPath, undefined);
          assert.deepEqual(result.platformPath, windowsProjPath);
          assert.deepEqual(result.windowsEnvPath, windowsProjPath);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should detect native-windows 10 when below root', function (done) {
        var pathBelowWindows = path.resolve(windowsProjPath, 'js');
        process.chdir(pathBelowWindows);
        windowsPlatform.detect()
        .then(function (result) {
          assert.notEqual(result, undefined);
          assert.deepEqual(result.appName, 'App2');
          assert.deepEqual(result.platformId, 'native-windows10');
          assert.deepEqual(result.platform, 'windows');
          assert.deepEqual(result.configXMLPath, undefined);
          assert.deepEqual(result.platformPath, windowsProjPath);
          assert.deepEqual(result.windowsEnvPath, windowsProjPath);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not detect native-windows 10 when outside project', function (done) {
        process.chdir(__dirname);
        windowsPlatform.detect()
        .then(function () {
          assert(false, 'It should not have detected native-windows10 outside project');
        }, function () {
          assert(true, 'Should not have detected a project');
          done();
        }).catch(function (err) {
          done(err);
        });
    });
  }),

  /**Folder structure B is when the .sln file is at a top level next to the app Folder
  /*  Dir structure
      Windows10NativeProjectAltStructure/
        App2.sln
        App2/
          package.appxmanifest
          ...app files...
  */
  describe('should detect windows10 with folder structure B', function () {
    var fixturePath,
        nativeProjectFixture,
        windowsProjPath,
        windowsEnvPath;

    before(function() {
        fixturePath = path.resolve(__dirname, '..', 'fixtures');
        nativeProjectFixture = getTestFixture('Windows10NativeProjectAltStructure.zip', fixturePath);

        windowsProjPath = path.resolve(fixturePath, 'Windows10NativeProjectAltStructure');
        windowsEnvPath = path.resolve(windowsProjPath, 'App2');
    });

    beforeEach(function() {
      nativeProjectFixture.clean();
    });

    after(function() {
      nativeProjectFixture.delete();
    });

    it('should detect native-windows 10 platform at root of project', function (done) {
        process.chdir(windowsProjPath);
        windowsPlatform.detect()
        .then(function (result) {
          assert.notEqual(result, undefined);
          assert.deepEqual(result.appName, 'App2');
          assert.deepEqual(result.platformId, 'native-windows10');
          assert.deepEqual(result.platform, 'windows');
          assert.deepEqual(result.configXMLPath, undefined);
          assert.deepEqual(result.platformPath, windowsProjPath);
          assert.deepEqual(result.windowsEnvPath, windowsEnvPath);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should detect native-windows 10 when below root, in app folder', function (done) {
        var pathBelowWindows = path.resolve(windowsEnvPath);
        process.chdir(pathBelowWindows);
        windowsPlatform.detect()
        .then(function (result) {
          assert.notEqual(result, undefined);
          assert.deepEqual(result.appName, 'App2');
          assert.deepEqual(result.platformId, 'native-windows10');
          assert.deepEqual(result.platform, 'windows');
          assert.deepEqual(result.configXMLPath, undefined);
          assert.deepEqual(result.platformPath, windowsProjPath);
          assert.deepEqual(result.windowsEnvPath, windowsEnvPath);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should detect native-windows 10 when further below root, below app folder', function (done) {
        var pathBelowWindows = path.resolve(windowsEnvPath, 'js');
        process.chdir(pathBelowWindows);
        windowsPlatform.detect()
        .then(function (result) {
          assert.notEqual(result, undefined);
          assert.deepEqual(result.appName, 'App2');
          assert.deepEqual(result.platformId, 'native-windows10');
          assert.deepEqual(result.platform, 'windows');
          assert.deepEqual(result.configXMLPath, undefined);
          assert.deepEqual(result.platformPath, windowsProjPath);
          assert.deepEqual(result.windowsEnvPath, windowsEnvPath);
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not detect native-windows 10 when outside project', function (done) {
        process.chdir(__dirname);
        windowsPlatform.detect()
        .then(function () {
          assert(false, 'It should not have detected native-windows10 outside project');
        }, function () {
          assert(true, 'Should not have detected a project');
          done();
        }).catch(function (err) {
          done(err);
        });
    }),

    it('should not detect native-windows 10 when appxmanifest does not exist', function (done) {
        fs.unlinkSync(path.resolve(windowsProjPath, 'App2', 'package.appxmanifest'));
        process.chdir(windowsProjPath);
        windowsPlatform.detect()
        .then(function () {
          assert(false, 'It should not have detected native-windows10 outside project');
        }, function () {
          assert(true, 'Should not have detected a project');
          done();
        }).catch(function (err) {
          done(err);
        });
    });
  });
});
