/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var fs = require('fs');
var path = require('path');
//private packages
var windowsPlatform = require('../../lib/nativeWindows10');
var reswAPI = require('../../lib/nativeWindows/process-properties-file/resw-api');
var getTestFixture = require('mdo-test-resources');

var validateFileOrFolderExists = function (path) {
    var result = true;
    try {
        fs.statSync(path);
    } catch (err) {
        result = false;
    }

    return result;
};

var fixturePath,
    nativeProjectFixture,
    propertyFilePath,
    windowsProjPath;

describe('platform-native-windows test process properties file', function () {
  before(function() {
      fixturePath = path.resolve(__dirname, '..', 'fixtures');
      nativeProjectFixture = getTestFixture('Windows10NativeProject.zip', fixturePath);

      nativeProjectFixture.clean();
      
      windowsProjPath = path.resolve(fixturePath, 'Windows10NativeProject');
      propertyFilePath = path.join(windowsProjPath, 'strings', 'mfpclient.resw');
  });

  after(function() {
      nativeProjectFixture.delete();
  });

  it('should build mfpclient.properties when not present', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(windowsProjPath), 'Windows10NativeProject should exist');
      assert.deepEqual(false, validateFileOrFolderExists(propertyFilePath), 'mfpclient.properties should not exist yet');

      var platformObj = {
        platformPath: windowsProjPath,
        windowsEnvPath: windowsProjPath
      };
      var serverObj = {
          protocol: 'http',
          host: '9.12.34.56',
          port: '5050',
      };
      var runtime = 'myRuntime';

      windowsPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should exist now');
        new reswAPI(propertyFilePath)
        .then(function (mfpClientData) {
          assert.deepEqual(mfpClientData.getDataEntry('wlServerProtocol'), 'http');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerHost'), '9.12.34.56');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerPort'), '5050');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerContext'), '/myRuntime/');
          assert.deepEqual(mfpClientData.getDataEntry('languagePreferences'), 'en');
          assert.deepEqual(mfpClientData.getDataEntry('wlAppId'), '');
          done();
        });
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update mfpclient.properties when file exists and server info is different', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(windowsProjPath), 'Windows10NativeProject should exist');
      assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should exist');

      var platformObj = {
        platformPath: windowsProjPath,
        windowsEnvPath: windowsProjPath
      };
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: '9090',
      };
      var runtime = 'altRuntime';

      windowsPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should exist now');
        new reswAPI(propertyFilePath)
        .then(function (mfpClientData) {
          assert.deepEqual(mfpClientData.getDataEntry('wlServerProtocol'), 'https');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerHost'), '1.2.3.4.5');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerPort'), '9090');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerContext'), '/altRuntime/');
          assert.deepEqual(mfpClientData.getDataEntry('languagePreferences'), 'en');
          assert.deepEqual(mfpClientData.getDataEntry('wlAppId'), '');
          done();
        });
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should update port when it is a Number', function (done) {
      assert.deepEqual(true, validateFileOrFolderExists(windowsProjPath), 'Windows10NativeProject should exist');
      assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should exist');

      var platformObj = {
        platformPath: windowsProjPath,
        windowsEnvPath: windowsProjPath
      };
      var serverObj = {
          protocol: 'https',
          host: '1.2.3.4.5',
          port: 9010,
      };
      var runtime = 'altRuntime';

      windowsPlatform.processPropertiesFile(platformObj, serverObj, runtime)
      .then(function () {
        assert.deepEqual(true, validateFileOrFolderExists(propertyFilePath), 'mfpclient.resw should exist now');
        new reswAPI(propertyFilePath)
        .then(function (mfpClientData) {
          assert.deepEqual(mfpClientData.getDataEntry('wlServerProtocol'), 'https');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerHost'), '1.2.3.4.5');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerPort'), '9010');
          assert.deepEqual(mfpClientData.getDataEntry('wlServerContext'), '/altRuntime/');
          assert.deepEqual(mfpClientData.getDataEntry('languagePreferences'), 'en');
          assert.deepEqual(mfpClientData.getDataEntry('wlAppId'), '');
          done();
        });
      }).catch(function (err) {
        done(err);
      });
  });
});
