/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var path = require('path');
//private
var errors = require('mdo-errors');
var detectPlatform = require('../').detectPlatform;
var getTestFixture = require('mdo-test-resources');

global.MDO_MAIN_EXTENSIONS_PATH = path.resolve(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

var fixturePath,
    cordovaProject,
    cordovaProjPath,
    originalCwd;

describe('detectplatform node module', function () {
  before(function() {
    //Cordova
    fixturePath = path.resolve(__dirname, 'fixtures');
    cordovaProject = getTestFixture('CordovaProject.zip', fixturePath);
    cordovaProject.clean();
    cordovaProjPath = path.resolve(fixturePath, 'CordovaProject');
    originalCwd = process.cwd();
  });

  after(function() {
    process.chdir(originalCwd);
    cordovaProject.delete();
  });

  it('should detect a single cordova platform if available', function (done) {
      process.chdir(cordovaProjPath);

      detectPlatform('cordova-ios')
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result.isCordova, true);
        assert.deepEqual(result.rootDir, cordovaProjPath);
        assert.deepEqual(result.platformId, 'cordova-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  //So the platforms folder is where the logic for detection is split, are you above or below
  //So this is a edge case that checks that at that mid section it still detects the desired platform
  it('should detect a single platform inside cordova platform directory', function (done) {
      var iosCordovaProjPath = path.resolve(cordovaProjPath, 'platforms');
      var rootDir = path.resolve(cordovaProjPath);
      process.chdir(iosCordovaProjPath);

      detectPlatform('cordova-ios')
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result.isCordova, true);
        assert.deepEqual(result.rootDir, rootDir);
        assert.deepEqual(result.platformId, 'cordova-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single platform below cordova platform directory', function (done) {
      var iosCordovaProjPath = path.resolve(cordovaProjPath, 'platforms', 'ios');
      var rootDir = path.resolve(cordovaProjPath);
      process.chdir(iosCordovaProjPath);

      detectPlatform('cordova-ios')
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result.isCordova, true);
        assert.deepEqual(result.rootDir, rootDir);
        assert.deepEqual(result.platformId, 'cordova-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single platform further below cordova platform directory', function (done) {
      var iosCordovaProjPath = path.resolve(cordovaProjPath, 'platforms', 'ios', 'HelloCordova');
      var rootDir = path.resolve(cordovaProjPath);
      process.chdir(iosCordovaProjPath);

      detectPlatform('cordova-ios')
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, true, 'result should be Cordova');
        assert.deepEqual(result.rootDir, rootDir, 'rootDir should be root level of project');
        assert.deepEqual(result.platformId, 'cordova-ios', 'platformId should be cordova-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should not detect a single platform if not available', function (done) {
      var doesNotExist = path.resolve(__dirname);
      process.chdir(doesNotExist);

      var noPlatformErr = errors.getError('NOPLATFORMDETECTED', 'cordova-ios');
      detectPlatform('cordova-ios')
      .then(function () {
        assert(false, 'should not have detected a platform');
      }, function (err) {
        assert.deepEqual(err.description, noPlatformErr.description);
        assert.deepEqual(err.errno, noPlatformErr.errno);
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should reject if no extension points are present', function (done) {
      var originalExtensionPath = global.MDO_MAIN_EXTENSIONS_PATH;
      global.MDO_MAIN_EXTENSIONS_PATH = path.resolve(__dirname, '..', 'node_modules');

      var noplatforms = path.resolve(__dirname);
      process.chdir(noplatforms);

      var noExtensionsErr = errors.getError('NOEXTENSIONSFOUND', 'mdo.platforms');

      detectPlatform('cordova-ios')
      .then(function () {
        assert(false, 'It should not have detected any platforms outside project');
      }, function (err) {
        assert.deepEqual(err.description, noExtensionsErr.description);
        assert.deepEqual(err.errno, noExtensionsErr.errno);
        global.MDO_MAIN_EXTENSIONS_PATH = originalExtensionPath;
        done();
      }).catch(function (err) {
        global.MDO_MAIN_EXTENSIONS_PATH = originalExtensionPath;
        done(err);
      });
  });
});
