/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var glob = require('glob');
var path = require('path');
var rimraf = require('rimraf');
//private
var errors = require('mdo-errors');
var detectPlatforms = require('../').detectPlatforms;
var getTestFixture = require('mdo-test-resources');

global.MDO_MAIN_EXTENSIONS_PATH = path.resolve(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

var fixturePath,
    cordovaProject,
    cordovaProjPath,
    androidProject,
    androidProjPath,
    iosProject,
    iosProjPath,
    windowsProject,
    windowsProjPath,
    windows8Project,
    windows8ProjPath,
    windows8AltProject,
    windows8AltProjPath,
    originalCwd;

describe('detectplatforms node module', function () {
  before(function() {
    originalCwd = process.cwd();
    //Cordova
    fixturePath = path.resolve(__dirname, 'fixtures');
    cordovaProject = getTestFixture('CordovaProject.zip', fixturePath);
    cordovaProject.clean();
    cordovaProjPath = path.resolve(fixturePath, 'CordovaProject');

    //Android
    androidProject = getTestFixture('AndroidNativeProject.zip', fixturePath);
    androidProject.clean();
    androidProjPath = path.resolve(fixturePath, 'AndroidNativeProject');

    //IOS
    iosProject = getTestFixture('IOSNativeProject.zip', fixturePath);
    iosProject.clean();
    iosProjPath = path.resolve(fixturePath, 'IOSNativeProject');

    //Windows
    windowsProject = getTestFixture('Windows10NativeProject.zip', fixturePath);
    windowsProject.clean();
    windowsProjPath = path.resolve(fixturePath, 'Windows10NativeProject', 'App2');

    //Windows8
    windows8Project = getTestFixture('Windows8NativeProject.zip', fixturePath);
    windows8Project.clean();
    windows8ProjPath = path.resolve(fixturePath, 'Windows8NativeProject', 'BlankSharedApp1');

    //Windows8Alt
    windows8AltProject = getTestFixture('Windows8AltNativeProject.zip', fixturePath);
    windows8AltProject.clean();
    windows8AltProjPath = path.resolve(fixturePath, 'Windows8AltNativeProject', 'Compression');

  });

  after(function() {
    process.chdir(originalCwd);
    cordovaProject.delete();
    androidProject.delete();
    iosProject.delete();
    windowsProject.delete();
    windows8Project.delete();
    windows8AltProject.delete();
  });

  it('should detect a multiple cordova platforms if available', function (done) {
      process.chdir(cordovaProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result.isCordova, true);
        assert.deepEqual(result.rootDir, cordovaProjPath);
        assert.deepEqual(result.platforms.length, 6);
        assert.deepEqual(result.platforms[0].platformId, 'cordova-android');
        assert.deepEqual(result.platforms[1].platformId, 'cordova-browser');
        assert.deepEqual(result.platforms[2].platformId, 'cordova-ios');
        assert.deepEqual(result.platforms[3].platformId, 'cordova-windows10');
        assert.deepEqual(result.platforms[4].platformId, 'cordova-windows8');
        assert.deepEqual(result.platforms[5].platformId, 'cordova-windows8phone');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  //So the platforms folder is where the logic for detection is split, are you above or below
  //So this is a edge case that checks that at that mid section it still detects the desired platform
  it('should detect multiple platforms inside cordova platform directory', function (done) {
      var cordovaProjPathBelow = path.resolve(cordovaProjPath, 'platforms');
      var rootDir = path.resolve(cordovaProjPath);
      process.chdir(cordovaProjPathBelow);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result.isCordova, true);
        assert.deepEqual(result.rootDir, rootDir);
        assert.deepEqual(result.platforms.length, 6);
        assert.deepEqual(result.platforms[0].platformId, 'cordova-android');
        assert.deepEqual(result.platforms[1].platformId, 'cordova-browser');
        assert.deepEqual(result.platforms[2].platformId, 'cordova-ios');
        assert.deepEqual(result.platforms[3].platformId, 'cordova-windows10');
        assert.deepEqual(result.platforms[4].platformId, 'cordova-windows8');
        assert.deepEqual(result.platforms[5].platformId, 'cordova-windows8phone');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single platform below cordova platform directory', function (done) {
      var iosCordovaProjPath = path.resolve(cordovaProjPath, 'platforms', 'ios');
      var rootDir = path.resolve(cordovaProjPath);
      process.chdir(iosCordovaProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result.isCordova, true);
        assert.deepEqual(result.rootDir, rootDir);
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'cordova-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single platform further below cordova platform directory', function (done) {
      var iosCordovaProjPath = path.resolve(cordovaProjPath, 'platforms', 'ios', 'HelloCordova');
      var rootDir = path.resolve(cordovaProjPath);
      process.chdir(iosCordovaProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, true, 'result should be Cordova');
        assert.deepEqual(result.rootDir, rootDir, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'cordova-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single native ios platform when inside ios project', function (done) {
      process.chdir(iosProjPath);

      detectPlatforms()
      .then(function (result) {
              assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should be not Cordova');
        assert.deepEqual(result.rootDir, iosProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-ios');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single native android platform when inside android project', function (done) {
      process.chdir(androidProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should be Cordova');
        assert.deepEqual(result.rootDir, androidProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-android');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a single native windows10 platform when inside windows10 project', function (done) {
      process.chdir(windowsProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windowsProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows10');
        assert.deepEqual(result.platforms[0].windowsEnvPath, windowsProjPath, 'should have windowsEnvPath pointing to App2 folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a two native windows8 platforms when inside windows8 project', function (done) {
      process.chdir(windows8ProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windows8ProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 2);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows8');
        assert.deepEqual(result.platforms[0].windowsEnvPath, path.resolve(windows8ProjPath, 'BlankSharedApp1', 'BlankSharedApp1.Windows'), 'should have windowsEnvPath pointing to BlankSharedApp1.Windows folder');
        assert.deepEqual(result.platforms[1].platformId, 'native-windows8phone');
        assert.deepEqual(result.platforms[1].windowsEnvPath, path.resolve(windows8ProjPath, 'BlankSharedApp1', 'BlankSharedApp1.WindowsPhone'), 'should have windowsEnvPath pointing to BlankSharedApp1.WindowsPhone folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect a two native windows8 platforms when inside windows8 project (alternate folder structure)', function (done) {
      process.chdir(windows8AltProjPath);

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windows8AltProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 2);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows8');
        assert.deepEqual(result.platforms[0].windowsEnvPath, path.resolve(windows8AltProjPath, 'Windows'), 'should have windowsEnvPath pointing to BlankSharedApp1.Windows folder');
        assert.deepEqual(result.platforms[1].platformId, 'native-windows8phone');
        assert.deepEqual(result.platforms[1].windowsEnvPath, path.resolve(windows8AltProjPath, 'WindowsPhone'), 'should have windowsEnvPath pointing to BlankSharedApp1.WindowsPhone folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should only detect native windows8 Windows platforms when inside windows8 Windows folder', function (done) {
      var windowsFolder = glob.sync('**/*Windows', {cwd: windows8ProjPath});
      assert.deepEqual(windowsFolder.length, 1, 'Windows folder should exist');
      process.chdir(path.join(windows8ProjPath, windowsFolder[0]));

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windows8ProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows8');
        assert.deepEqual(result.platforms[0].windowsEnvPath, path.resolve(windows8ProjPath, 'BlankSharedApp1', 'BlankSharedApp1.Windows'), 'should have windowsEnvPath pointing to BlankSharedApp1.Windows folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect only detect native windows8 WindowsPhone platforms when inside windows8 WindowsPhone folder', function (done) {
      var windowsPhoneFolder = glob.sync('**/*WindowsPhone', {cwd: windows8ProjPath});
      assert.deepEqual(windowsPhoneFolder.length, 1, 'WindowsPhone folder should exist');
      process.chdir(path.join(windows8ProjPath, windowsPhoneFolder[0]));

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windows8ProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows8phone');
        assert.deepEqual(result.platforms[0].windowsEnvPath, path.resolve(windows8ProjPath, 'BlankSharedApp1', 'BlankSharedApp1.WindowsPhone'), 'should have windowsEnvPath pointing to BlankSharedApp1.WindowsPhone folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect only detect native windows8 WindowsPhone platform when Windows folder is missing', function (done) {
      process.chdir(windows8ProjPath);

      var windowsFolder = glob.sync('**/*Windows', {cwd: windows8ProjPath});
      assert.deepEqual(windowsFolder.length, 1, 'Windows folder should exist');
      rimraf.sync(windowsFolder[0]);
      windowsFolder = glob.sync('**/*Windows', {cwd: windows8ProjPath});
      assert.deepEqual(windowsFolder.length, 0, 'Windows folder should have been deleted');

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windows8ProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows8phone');
        assert.deepEqual(result.platforms[0].windowsEnvPath, path.resolve(windows8ProjPath, 'BlankSharedApp1', 'BlankSharedApp1.WindowsPhone'), 'should have windowsEnvPath pointing to BlankSharedApp1.WindowsPhone folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should detect only detect native windows8 Windows platform when WindowsPhone folder is missing', function (done) {
      //We need to clean our fixture to bring back our Windows folder
      windows8Project.clean();
      process.chdir(windows8ProjPath);

      var windowsPhoneFolder = glob.sync('**/*WindowsPhone', {cwd: windows8ProjPath});
      assert.deepEqual(windowsPhoneFolder.length, 1, 'WindowsPhone folder should exist');
      rimraf.sync(windowsPhoneFolder[0]);
      windowsPhoneFolder = glob.sync('**/*WindowsPhone', {cwd: windows8ProjPath});
      assert.deepEqual(windowsPhoneFolder.length, 0, 'WindowsPhone folder should have been deleted');

      detectPlatforms()
      .then(function (result) {
        assert.notEqual(result, undefined, 'result should not be undefined');
        assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
        assert.deepEqual(result.rootDir, windows8ProjPath, 'rootDir should be root level of project');
        assert.deepEqual(result.platforms.length, 1);
        assert.deepEqual(result.platforms[0].platformId, 'native-windows8');
        assert.deepEqual(result.platforms[0].windowsEnvPath, path.resolve(windows8ProjPath, 'BlankSharedApp1', 'BlankSharedApp1.Windows'), 'should have windowsEnvPath pointing to BlankSharedApp1.Windows folder');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should not detect a single platform if not available', function (done) {
      var noplatforms = path.resolve(__dirname);
      process.chdir(noplatforms);

      var noPlatformsErr = errors.getError('NOPLATFORMSDETECTED');
      detectPlatforms()
      .then(function () {
        assert(false, 'It should not have detected any platforms outside project');
      }, function (err) {
        assert.deepEqual(err.description, noPlatformsErr.description);
        assert.deepEqual(err.errno, noPlatformsErr.errno);
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should reject if no extension points are present', function (done) {
      var originalExtensionPath = global.MDO_MAIN_EXTENSIONS_PATH;
      global.MDO_MAIN_EXTENSIONS_PATH = path.resolve(__dirname, '..', 'node_modules');

      var noplatforms = path.resolve(__dirname);
      process.chdir(noplatforms);

      var noExtensionsErr = errors.getError('NOEXTENSIONSFOUND', 'mdo.platforms');

      detectPlatforms()
      .then(function () {
        assert(false, 'It should not have detected any platforms outside project');
      }, function (err) {
        assert.deepEqual(err.description, noExtensionsErr.description);
        assert.deepEqual(err.errno, noExtensionsErr.errno);
        global.MDO_MAIN_EXTENSIONS_PATH = originalExtensionPath;
        done();
      }).catch(function (err) {
        global.MDO_MAIN_EXTENSIONS_PATH = originalExtensionPath;
        done(err);
      });
  }),

  //Tests related to defect of platforms within platforms
  it('should detect IOS project inside Android project', function (done) {
    var mutatedFixturePath = path.resolve(fixturePath, 'mutation');
    var mutatedAndroidProject = getTestFixture('AndroidNativeProject.zip', mutatedFixturePath);
    mutatedAndroidProject.clean();
    var mutatedAndroidProjPath = path.resolve(mutatedFixturePath, 'AndroidNativeProject', 'gradle');
    var mutatedIOSProject = getTestFixture('IOSNativeProject.zip', mutatedAndroidProjPath);
    mutatedIOSProject.clean();
    var mutatedIOSProjPath = path.resolve(mutatedAndroidProjPath, 'IOSNativeProject');
    process.chdir(mutatedIOSProjPath);

    detectPlatforms()
    .then(function (result) {
      assert.notEqual(result, undefined, 'result should not be undefined');
      assert.deepEqual(result.isCordova, false, 'result should be not Cordova');
      assert.deepEqual(result.rootDir, mutatedIOSProjPath, 'rootDir should be root level of project');
      assert.deepEqual(result.platforms.length, 1);
      assert.deepEqual(result.platforms[0].platformId, 'native-ios');
      mutatedAndroidProject.delete();
      mutatedIOSProject.delete();
      done();
    }).catch(function (err) {
      mutatedAndroidProject.delete();
      mutatedIOSProject.delete();
      done(err);
    });
  }),

  it('should detect Windows10 project inside IOS project inside Android project', function (done) {
    var mutatedFixturePath = path.resolve(fixturePath, 'mutation');
    var mutatedAndroidProject = getTestFixture('AndroidNativeProject.zip', mutatedFixturePath);
    mutatedAndroidProject.clean();
    var mutatedAndroidProjPath = path.resolve(mutatedFixturePath, 'AndroidNativeProject', 'gradle');
    var mutatedIOSProject = getTestFixture('IOSNativeProject.zip', mutatedAndroidProjPath);
    mutatedIOSProject.clean();
    var mutatedIOSProjPath = path.resolve(mutatedAndroidProjPath, 'IOSNativeProject', 'TestNativeProject', 'Base.lproj');

    var mutatedWindows10Project = getTestFixture('Windows10NativeProject.zip', mutatedIOSProjPath);
    mutatedWindows10Project.clean();
    var mutatedWindows10ProjPath = path.resolve(mutatedIOSProjPath, 'Windows10NativeProject', 'App2');
    process.chdir(mutatedWindows10ProjPath);

    detectPlatforms()
    .then(function (result) {
      assert.notEqual(result, undefined, 'result should not be undefined');
      assert.deepEqual(result.isCordova, false, 'result should not be Cordova');
      assert.deepEqual(result.rootDir, mutatedWindows10ProjPath, 'rootDir should be root level of project');
      assert.deepEqual(result.platforms.length, 1);
      assert.deepEqual(result.platforms[0].platformId, 'native-windows10');
      assert.deepEqual(result.platforms[0].windowsEnvPath, mutatedWindows10ProjPath, 'should have windowsEnvPath pointing to App2 folder');
      mutatedAndroidProject.delete();
      mutatedIOSProject.delete();
      mutatedWindows10Project.delete();
      done();
    }).catch(function (err) {
      mutatedAndroidProject.delete();
      mutatedIOSProject.delete();
      mutatedWindows10Project.delete();
      done(err);
    });
  }),

  it('should detect Cordova project inside Windows10 inside IOS inside Android project', function (done) {
    var mutatedFixturePath = path.resolve(fixturePath, 'mutation');
    var mutatedAndroidProject = getTestFixture('AndroidNativeProject.zip', mutatedFixturePath);
    mutatedAndroidProject.clean();
    var mutatedAndroidProjPath = path.resolve(mutatedFixturePath, 'AndroidNativeProject', 'gradle');
    var mutatedIOSProject = getTestFixture('IOSNativeProject.zip', mutatedAndroidProjPath);
    mutatedIOSProject.clean();
    var mutatedIOSProjPath = path.resolve(mutatedAndroidProjPath, 'IOSNativeProject', 'TestNativeProject', 'Base.lproj');

    var mutatedWindows10Project = getTestFixture('Windows10NativeProject.zip', mutatedIOSProjPath);
    mutatedWindows10Project.clean();
    var mutatedWindows10ProjPath = path.resolve(mutatedIOSProjPath, 'Windows10NativeProject', 'App2');

    var mutatedCordovaProject = getTestFixture('CordovaProject.zip', mutatedWindows10ProjPath);
    mutatedCordovaProject.clean();
    var mutatedCordovaProjPath = path.resolve(mutatedWindows10ProjPath, 'CordovaProject');

    process.chdir(mutatedCordovaProjPath);

    detectPlatforms()
    .then(function (result) {
      assert.notEqual(result, undefined);
      assert.deepEqual(result.isCordova, true);
      assert.deepEqual(result.rootDir, mutatedCordovaProjPath);
      assert.deepEqual(result.platforms.length, 6);
      assert.deepEqual(result.platforms[0].platformId, 'cordova-android');
      assert.deepEqual(result.platforms[1].platformId, 'cordova-browser');
      assert.deepEqual(result.platforms[2].platformId, 'cordova-ios');
      assert.deepEqual(result.platforms[3].platformId, 'cordova-windows10');
      assert.deepEqual(result.platforms[4].platformId, 'cordova-windows8');
      assert.deepEqual(result.platforms[5].platformId, 'cordova-windows8phone');
      mutatedAndroidProject.delete();
      mutatedIOSProject.delete();
      mutatedWindows10Project.delete();
      mutatedCordovaProject.delete();
      done();
    }).catch(function (err) {
      mutatedAndroidProject.delete();
      mutatedIOSProject.delete();
      mutatedWindows10Project.delete();
      mutatedCordovaProject.delete();
      done(err);
    });
  });
});
