/*
	Licensed Material - Property of IBM

	(C) Copyright 2015,2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';
/*jshint -W030 */

var assert = require('assert');
var path = require('path');
//private
var errors = require('mdo-errors');
var parsePlatform = require('../').parsePlatform;
var getTestFixture = require('mdo-test-resources');

global.MDO_MAIN_EXTENSIONS_PATH = path.resolve(__dirname, '..', '..', 'mfpdev-cli', 'node_modules');

var fixturePath,
    cordovaProject,
    cordovaProjPath,
    androidProject,
    androidProjPath;

describe('parsePlatform node module', function () {
  before(function() {
    fixturePath = path.resolve(__dirname, 'fixtures');
      //Cordova
    cordovaProject = getTestFixture('CordovaProject.zip', fixturePath);
    cordovaProject.clean();
    cordovaProjPath = path.resolve(fixturePath, 'CordovaProject');

      //Android
    androidProject = getTestFixture('AndroidNativeProject.zip', fixturePath);
    androidProject.clean();
    androidProjPath = path.resolve(fixturePath, 'AndroidNativeProject');
  });

  after(function() {
    cordovaProject.delete();
    androidProject.delete();
  });

  it('should parse a single platform (cordova) when available', function (done) {
      parsePlatform('cordova-ios', cordovaProjPath)
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.equal(result['app.description'], "A sample Apache Cordova application that responds to the deviceready event.");
        assert.deepEqual(result['app.dotId'], 'io.cordova.hellocordova');
        assert.deepEqual(result['app.mainfile'], 'index.html');
        assert.deepEqual(result['app.name'], 'HelloCordova');
        assert.deepEqual(result['app.version'], '0.0.1');
        assert.deepEqual(result['author.email'], 'dev@cordova.apache.org');
        assert.deepEqual(result['author.href'], 'http://cordova.io');
        assert.deepEqual(result['author.name'], 'Apache Cordova Team');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should parse a single platform (native-android) when available', function (done) {
      parsePlatform('native-android', androidProjPath)
      .then(function (result) {
        assert.notEqual(result, undefined);
        assert.deepEqual(result['app.dotId'], 'test.ibm.com.testnativeproject');
        assert.deepEqual(result['app.name'], 'TestNativeProject');
        assert.deepEqual(result['app.version'], '1.0');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should not parse when provided path without config.xml', function (done) {
      var invalidPath = path.resolve(__dirname);

      parsePlatform('cordova-ios', invalidPath)
      .then(function () {
        assert(false, 'should not have executed success function');
      }, function () {
        assert(true, 'successfully rejected invalid platformId');
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should not parse when provided invalid platformId', function (done) {

    var noPlatformErr = errors.getError('NOPLATFORMDETECTED', 'cordova-noExist');

      parsePlatform('cordova-noExist', cordovaProjPath)
      .then(function () {
        assert(false, 'should not have executed success function');
      }, function (err) {
        assert.deepEqual(err.description, noPlatformErr.description);
        assert.deepEqual(err.errno, noPlatformErr.errno);
        done();
      }).catch(function (err) {
        done(err);
      });
  }),

  it('should reject if no extension points are present', function (done) {
      var originalExtensionPath = global.MDO_MAIN_EXTENSIONS_PATH;
      global.MDO_MAIN_EXTENSIONS_PATH = path.resolve(__dirname, '..', 'node_modules');

      var noExtensionsErr = errors.getError('NOEXTENSIONSFOUND', 'mdo.platforms');

      parsePlatform('cordova-ios', cordovaProjPath)
      .then(function () {
        assert(false, 'It should not have detected any platforms outside project');
      }, function (err) {
        assert.deepEqual(err.description, noExtensionsErr.description);
        assert.deepEqual(err.errno, noExtensionsErr.errno);
        global.MDO_MAIN_EXTENSIONS_PATH = originalExtensionPath;
        done();
      }).catch(function (err) {
        global.MDO_MAIN_EXTENSIONS_PATH = originalExtensionPath;
        done(err);
      });
  });

});
